<?php
/**
 * @SWG\Post(path="/add_to_cart",
 *     summary="Add to card",
 *     consumes={"application/x-www-form-urlencoded"},
 *     @SWG\Parameter(
 *         description="Product id",
 *         in="formData",
 *         name="pID",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="Mil-PRF-16173",
 *     ),
 *     @SWG\Parameter(
 *         description="Product option id",
 *         in="formData",
 *         name="optID",
 *         required=false,
 *         type="string",
 *         format="string",
 *         default="41",
 *     ),
 *     @SWG\Parameter(
 *         name="clID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="0",
 *     ),
 *     @SWG\Parameter(
 *         description="Is guest (0 = Login user, 1 = Guest)",
 *         name="tIsGuest",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default="0",
 *     ),
 *     @SWG\Parameter(
 *         name="cartSessionID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Add to cart response",
 *         @SWG\Schema(
 *                ref="#/definitions/AddToCartFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                       "message": "Product added to cart.",
 *                       "data": {
 *                           "cartSessionID": "0ouxdiibecgfu0p40dyek42wg86wo4o04on22db7hn8mubuwfh52khg5xp5h5"
 *                       }
 *                   }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         @SWG\Schema(
 *                ref="#/definitions/ErrorResponse"
 *              ), 
 *         examples={
 *                   "application/json": {
 *                         "message": "Sub category list.",
 *                         "data": {}
 *                   }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = $_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('POST');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post);
    if ($response['status'] == 200) {
        
        if(isset($_SERVER['HTTP_TISGUEST']) && trim($_SERVER['HTTP_TISGUEST']) != ""){
            $tIsGuest = trim($_SERVER['HTTP_TISGUEST']);
            if($tIsGuest == 0 || $tIsGuest == ""){
                //User Authentication
                $response = Authentication::auth_user();
                if ($response['status'] != 200) {
                    _response($response);
                    return;
                }
            }
            $requiredFields = [
                'pID' => 'required'
            ];
            $response = $obj->fieldValidation($post, $requiredFields);
            if ($response['status'] == 200) {
                $model = new Cart();
                $response = $model->add_to_cart($post,$tIsGuest);
            }
        }else{
            //guest param is required
            $response = ['status' => false, 'message' => 'tIsGuest key is required.'];
        }
    }
}
_response($response);
