<?php
/**
 * @SWG\Post(path="/ask_question",
 *     summary="Ask a question",
 *     consumes={"application/x-www-form-urlencoded"},
 *     @SWG\Parameter(
 *         description="Name",
 *         in="formData",
 *         name="vName",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="test",
 *     ),
 *     @SWG\Parameter(
 *         description="Email address",
 *         in="formData",
 *         name="vEmail",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="test@gmail.com",
 *     ),
 *     @SWG\Parameter(
 *         description="Comment",
 *         in="formData",
 *         name="txComment",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="Hello world", 
 *     ),
 *     @SWG\Parameter(
 *         description="Product id",
 *         in="formData",
 *         name="pID",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="VCI-4x1000",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Ask a question response",
 *         @SWG\Schema(
 *                ref="#/definitions/AskQuestionFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                           "message": "Thank you, your query has been sent and we will get back to you as soon as possible.\nPlease click below to close this window.",
 *                   }
 *           },
 *     ),
 *   @SWG\Response(
 *         response = 400,
 *         description = "Confilct",   
 *         @SWG\Schema(
 *                ref="#/definitions/ErrorResponse"
 *              ),        
 *         examples={
 *                   "application/json": {
 *                           "message": "vName is required"
 *                       }
 *                   },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = $_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('POST');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post);
    if ($response['status'] == 200) {
        //Validation rules
        $requiredFields = [
            'vName' => 'required',
            'vEmail' => 'required|email',
            'txComment' => 'required',
            'pID' => 'required'
        ];
        $response = $obj->fieldValidation($post, $requiredFields);
        if ($response['status'] == 200) {
            $model = new Customerlogin();
            $response = $model->askQuestion($post);
        }
    }
}
_response($response);
