<?php
/**
 * @SWG\Post(path="/cart_list",
 *     summary="Shopping cart list",
 *     @SWG\Parameter(
 *         description="Country id",
 *         in="formData",
 *         name="countryID",
 *         required=false,
 *         type="integer",
 *         format="integer",
 *         default="1",
 *     ),
 *     @SWG\Parameter(
 *         description="Zip code",
 *         in="formData",
 *         name="zipCode",
 *         required=false,
 *         type="string",
 *         format="string",
 *         default="90001",
 *     ),
 *     @SWG\Parameter(
 *         name="clID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="586",
 *     ),
 *     @SWG\Parameter(
 *         description="Is guest (0 = Login user, 1 = Guest)",
 *         name="tIsGuest",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default="0",
 *     ),
 *     @SWG\Parameter(
 *         name="cartSessionID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Cart list response",
 *         @SWG\Schema(
 *                ref="#/definitions/CartListFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                       "message": "Cart list",
 *                       "data": {
 *                           "dSubTotal": 1142.95,
 *                           "dShippingHandlingCharge": 57.99,
 *                           "dGrandTotal": 1200.94,
 *                           "dGrandTotalWithoutShipping": 1021.1600000000001,
 *                           "cartList": {
 *                               {
 *                                   "cartID": 22873,
 *                                   "cartProdID": "KPR-rust-remover-55gal",
 *                                   "cartProdName": "KPR Rust Remover 55 Gal",
 *                                   "optionName": "Nox-Rust Sizes - Six Aerosol Cans -Ground ONLY",
 *                                   "cartProdPrice": 791,
 *                                   "cartQuantity": 1,
 *                                   "cartOrderID": 0,
 *                                   "pExemptions": 0,
 *                                   "pWeight": 0,
 *                                   "imageSrc": "//www.static.corrosionvci.com/prodimages/rust-remover-drum.jpg",
 *                                   "dTotalPrice": 791
 *                               },
 *                               {
 *                                   "cartID": 22874,
 *                                   "cartProdID": "vci9x12zip",
 *                                   "cartProdName": "VCI 9x12 Zipper Bags",
 *                                   "optionName": "Pallet Quantity - 24 Rolls @ $90/Roll",
 *                                   "cartProdPrice": 210,
 *                                   "cartQuantity": 1,
 *                                   "cartOrderID": 0,
 *                                   "pExemptions": 0,
 *                                   "pWeight": 47,
 *                                   "imageSrc": "//www.static.corrosionvci.com/prodimages/zipper-bags.jpg",
 *                                   "dTotalPrice": 210
 *                               }
 *                           }
 *                       }
 *                   }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         @SWG\Schema(
 *                ref="#/definitions/ErrorResponse"
 *              ), 
 *         examples={
 *                   "application/json": {
 *                         "message": "Cart list.",
 *                         "data": {}
 *                   }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = $_POST; //$_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('POST');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post, false);
    if ($response['status'] == 200) {
        if(isset($_SERVER['HTTP_TISGUEST']) && trim($_SERVER['HTTP_TISGUEST']) != ""){
            $tIsGuest = trim($_SERVER['HTTP_TISGUEST']);
            if($tIsGuest == 0 || $tIsGuest == ""){
                //User Authentication
                $response = Authentication::auth_user();
                if ($response['status'] != 200) {
                    _response($response);
                    return;
                }
            }
            $model = new Cart();
            $response = $model->cart_list($post,NULL,$tIsGuest);
        }else{
            //guest param is required
            $response = ['status' => false, 'message' => 'tIsGuest key is required.'];
        }
    }
}
_response($response);
