<?php

/**
 * @SWG\Get(path="/custom_quote_options",
 *     summary="Custom quote options list",
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Custom quote options list response",
 *         @SWG\Schema(
 *                ref="#/definitions/QuotationOptionFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                       "message": "Custom quote options list",
 *                       "data": {
 *                           "stateList": {
 *                               {
 *                                   "vName": "Alabama"
 *                               },
 *                               {
 *                                   "vName": "Alaska"
 *                               }
 *                           },
 *                           "customMadeList": {
 *                               {
 *                                   "vName": "VCI Paper-Ferrous"
 *                               },
 *                               {
 *                                   "vName": "VCI Paper-Multi-metal"
 *                               }
 *                           },
 *                           "thicknessList": {
 *                               {
 *                                   "vName": "2 Mil"
 *                               },
 *                               {
 *                                   "vName": "3 Mil"
 *                               }
 *                           },
 *                           "purchaseQtyList": {
 *                               {
 *                                   "vName": "250"
 *                               },
 *                               {
 *                                   "vName": "500"
 *                               }
 *                           },
 *                           "purchaseFrequencyList": {
 *                               {
 *                                   "vName": "Weekly"
 *                               },
 *                               {
 *                                   "vName": "Monthly"
 *                               }
 *                           },
 *                           "typeOfProductWantToProtect": {
 *                               {
 *                                   "vName": "Ferrous metals"
 *                               },
 *                               {
 *                                   "vName": "Cast iron"
 *                               }
 *                           },
 *                          "forDomesticOrExport": {
 *                              {
 *                                  "vName": "Domestic"
 *                              },
 *                              {
 *                                  "vName": "Export"
 *                              }
 *                          },
 *                          "preferredMethodOfCorrosionProtection": {
 *                              {
 *                                 "vName": "I don't know"
 *                              },
 *                              {
 *                                 "vName": "VCI Poly Bags"
 *                              }
 *                          },
 *                           "searchType": {
 *                               {
 *                                   "vKey": 1,
 *                                   "vName": "All words"
 *                               },
 *                               {
 *                                   "vKey": 2,
 *                                   "vName": "Any words"
 *                               },
 *                               {
 *                                   "vKey": 3,
 *                                   "vName": "Exact Phrase"
 *                               }
 *                           }
 *                     }
 *                }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         examples={
 *                   "application/json": {
 *                         "message": "Custom quote options list.",
 *                         "data": {}
 *                   }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = []; //$_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('GET');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post, false);
    if ($response['status'] == 200) {
        $model = new Customerlogin();
        $response = $model->customQuoteOptions();
    }
}
_response($response);
