<?php
/**
 * @SWG\Get(path="/generate_client_token",
 *     summary="Generate client token",
 *     @SWG\Parameter(
 *         description="Country id",
 *         in="query",
 *         name="countryID",
 *         required=false,
 *         type="integer",
 *         format="integer",
 *         default="",
 *     ),
 *     @SWG\Parameter(
 *         description="Zip code",
 *         in="query",
 *         name="zipCode",
 *         required=false,
 *         type="string",
 *         format="string",
 *         default="",
 *     ),
 *     @SWG\Parameter(
 *         description="State name",
 *         in="query",
 *         name="stateName",
 *         required=false,
 *         type="string",
 *         format="string",
 *         default="",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Parameter(
 *         name="clID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="586",
 *     ),
 *     @SWG\Parameter(
 *         description="Is guest (0 = Login user, 1 = Guest)",
 *         name="tIsGuest",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default="0",
 *     ),
 *     @SWG\Parameter(
 *         name="cartSessionID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="",
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Country list response",
 *         @SWG\Schema(
 *                ref="#/definitions/GenerateTokenFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                       {
 *                           "message": "Operation success",
 *                           "data": {
 *                               "clientToken": "",
 *                               "ordHandling": 4.95,
 *                               "fedexOptions": {
 *                                   {
 *                                      "key": "FedEx Ground&reg;",
 *                                      "days": "2 days",
 *                                      "price": 4137.4800000000005
 *                                   },
 *                                   {
 *                                      "key": "FedEx Express Saver&reg;",
 *                                      "days": "7 days",
 *                                      "price": 12237.36
 *                                   }
 *                           }
 *                       }
 *                   }
 *              }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         @SWG\Schema(
 *                ref="#/definitions/ErrorResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                         "message": "Country list.",
 *                         "data": {}
 *                   }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = $_GET; //$_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('GET');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post, false);
    if ($response['status'] == 200) {
        if(isset($_SERVER['HTTP_TISGUEST']) && trim($_SERVER['HTTP_TISGUEST']) != ""){
            $tIsGuest = trim($_SERVER['HTTP_TISGUEST']);
            if($tIsGuest == 0 || $tIsGuest == ""){
                //User Authentication
                $response = Authentication::auth_user();
                if ($response['status'] != 200) {
                    _response($response);
                    return;
                }
            }
            $model = new Customerlogin();
            $response = $model->generateClientToken($post);
        }else{
            //guest param is required
            $response = ['status' => false, 'message' => 'tIsGuest key is required.'];
        }
    }
}
_response($response);
