<?php
/**
 * @SWG\Post(path="/get_order_status",
 *     summary="Get order status",
 *     consumes={"application/x-www-form-urlencoded"},
 *     @SWG\Parameter(
 *         description="Order id",
 *         in="formData",
 *         name="ordID",
 *         required=true,
 *         type="integer",
 *         format="integer",
 *         default="5035",
 *     ),
 *     @SWG\Parameter(
 *         description="Order email",
 *         in="formData",
 *         name="ordEmail",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="abc@gmail.com",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Order status response",
 *         @SWG\Schema(
 *                ref="#/definitions/OrderStatusFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                           "message": "Track order status.",
 *                           "data": {
 *                               "ordStatus": 2,
 *                               "ordStatusDate": "2018-11-01 09:57:51",
 *                               "statPublic": "Awaiting Payment",
 *                               "ordTrackNum": 771361604037,
 *                           }
 *                  }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         examples={
 *                   "application/json": {
 *                   "message": "Product list",
 *                   "data": {}
 *               }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */

$response = [];
$post = $_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('POST');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post);
    if ($response['status'] == 200) {
        //Validation rules
        $requiredFields = [
            'ordID' => 'required',
            'ordEmail' => 'required|email',
        ];
        $response = $obj->fieldValidation($post, $requiredFields);
        if ($response['status'] == 200) {
            $model = new Order();
            $response = $model->getOrderStatus($post);
        }
    }
}
_response($response);
