<?php

/**
 * @SWG\Get(path="/nested_category_list",
 *     summary="Nested category list",
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Nested category list response",
 *         @SWG\Schema(
 *                ref="#/definitions/NestedCategoryListFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                       "message": "Nasted category list",
 *                       "data": {
 *                           {
 *                               "sectionID": 22,
 *                               "sectionName": "Specials",
 *                               "topSection": 0,
 *                               "rootSection": 1,
 *                               "sub_category": {}
 *                           },
 *                           {
 *                               "sectionID": 5,
 *                               "sectionName": "VCI Paper Industrial Packaging",
 *                               "topSection": 0,
 *                               "rootSection": 0,
 *                               "sub_category": {
 *                                   {
 *                                       "sectionID": 7,
 *                                       "sectionName": "Multi-Metal VCI Paper",
 *                                       "topSection": 5,
 *                                       "rootSection": 1,
 *                                       "sub_category": {}
 *                                   },
 *                                   {
 *                                       "sectionID": 3,
 *                                       "sectionName": "VCI Paper Non-Ferrous Metals",
 *                                       "topSection": 5,
 *                                       "rootSection": 0,
 *                                       "sub_category": {
 *                                           {
 *                                               "sectionID": 8,
 *                                               "sectionName": "Silver Saver Paper",
 *                                               "topSection": 3,
 *                                               "rootSection": 1,
 *                                               "sub_category": {}
 *                                           }
 *                                       }
 *                                   },
 *                                   {
 *                                       "sectionID": 78,
 *                                       "sectionName": "Versil_Pak",
 *                                       "topSection": 5,
 *                                       "rootSection": 0,
 *                                       "sub_category": {
 *                                           {
 *                                               "sectionID": 79,
 *                                               "sectionName": "Versil-Pak Grades",
 *                                               "topSection": 78,
 *                                               "rootSection": 1,
 *                                               "sub_category": {}
 *                                           }
 *                                       }
 *                                   }
 *                               }
 *                           }
 *                       }
 *                   }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         @SWG\Schema(
 *                ref="#/definitions/ErrorResponse"
 *              ),   
 *         examples={
 *                   "application/json": {
 *                         "message": "Nasted category list",
 *                         "data": {}
 *                   }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = []; //$_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('GET');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post, false);
    if ($response['status'] == 200) {
        $model = new Section();
        $response = $model->nested_category_list();
    }
}
_response($response);
