<?php
/**
 * @SWG\Delete(path="/remove_cart_product/{cartID}",
 *     summary="Product detail",
 *     consumes={"application/x-www-form-urlencoded"},
 *     @SWG\Parameter(
 *         name="cartID",
 *         in="path",
 *         required=true,
 *         type="string",
 *         default="22869",
 *     ),
 *     @SWG\Parameter(
 *         name="clID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="569",
 *     ),
 *     @SWG\Parameter(
 *         description="Is guest (0 = Login user, 1 = Guest)",
 *         name="tIsGuest",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default="0",
 *     ),
 *     @SWG\Parameter(
 *         name="cartSessionID",
 *         in="header",
 *         required=false,
 *         type="string",
 *         default="",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Remove item from cart",
 *         @SWG\Schema(
 *                ref="#/definitions/RemoveCartProductFinalResponse"
 *              ),
 *         examples={
 *                   "application/json": {
 *                           "message": "Item removed from cart."
 *                       }
 *          }
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         examples={
 *                   "application/json": {
 *                   "message": "Cart reference is invalid."
 *               }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('POST');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck([], false);
    if ($response['status'] == 200) {
        if(isset($_SERVER['HTTP_TISGUEST']) && trim($_SERVER['HTTP_TISGUEST']) != ""){
            $tIsGuest = trim($_SERVER['HTTP_TISGUEST']);
            if($tIsGuest == 0 || $tIsGuest == ""){
                //User Authentication
                $response = Authentication::auth_user();
                if ($response['status'] != 200) {
                    _response($response);
                    return;
                }
            }
            //Validation rules
            $post = [
                'cartID' => $_params
            ];
            $requiredFields = [
                'cartID' => 'required',
            ];
            $response = $obj->fieldValidation($post, $requiredFields);
            if ($response['status'] == 200) {
                $model = new Cart();
                $response = $model->removeCartProduct($post,$tIsGuest);
            }
        }else{
            //guest param is required
            $response = ['status' => false, 'message' => 'tIsGuest key is required.'];
        }
    }
}
_response($response);
