<?php

use Illuminate\Database\Eloquent\Model as Eloquent;
use Illuminate\Database\Capsule\Manager as DB;

class Cart extends Eloquent {

    protected $table = 'cart';
    protected $primaryKey = 'cartID';
    public $timestamps = false;

    public function product() {
        return $this->hasOne(Product::class, 'pID', 'cartProdID');
    }
    
    public function cart_option() {
        return $this->hasOne(Cartoption::class, 'coCartID', 'cartID');
    }

    public function add_to_cart($post,$tIsGuest) {
        $response = [];
        try {
            $user = ($tIsGuest != 1) ? Authentication::getuser() : [];
            $product = Product::where(['pID' => $post['pID']])->first();
            if (!empty($product)) {
                //check the guest have added product in cart
                $cartSessionID = "";
                if($tIsGuest == 1){
                    $cartSessionID = trim($_SERVER['HTTP_CARTSESSIONID']);
                    if($cartSessionID == ""){
                        do{
                            $cartSessionID = Common::generateRandomToken();
                            $g_cart = Cart::from('cart as c')
                                ->where('c.cartSessionID', $cartSessionID)
                                ->count();
                        }while($g_cart > 0);
                    }
                }
                
                $result = [
                    'cartSessionID' => $cartSessionID
                ];
                
                $list = Cart::from('cart as c')
                        ->where('c.cartProdID', $post['pID']);
                if($tIsGuest == 1){
                    $list->where('c.cartSessionID', $cartSessionID);
                }else{
                    $list->where('c.cartClientID', $user->clID);
                }
                $list->whereRaw('(c.cartOrderID = 0 OR c.cartOrderID IS NULL)');

                $option = [];
                $cart = [];
                if (isset($post['optID']) && trim($post['optID']) != "") {
                    $cart = $list->get();
                    $option = Option::from('options AS o')
                            ->selectRaw('o.optID,o.optName,o.optPriceDiff,o.optWeightDiff,o.optRegExp,og.optGrpName,og.optType,og.optFlags')
                            ->leftjoin('optiongroup AS og', 'og.optGrpID', '=', 'o.optGroup')
                            ->where(['optID' => $post['optID']])
                            ->first();
                    
                    if (empty($option)) {
                        return ['status' => false, 'message' => 'invalid_option'];
                    }
                } else {
                    $cart = $list->first();
                }
                
                if (empty($cart)) {
                    return $this->addProductToCart($post, $product, $user, $option,$tIsGuest,$cartSessionID);
                } else {
                    $cartOptions = [];
                    $flag = true;
                    if (!empty($option) && isset($post['optID']) && trim($post['optID']) != "") {
                        foreach ($cart as $key => $val) {
                            $cartOptions = Cartoption::where('coCartID', $val->cartID)
                                    ->where('coOptID', $post['optID'])
                                    ->first();
                            
                            if (!empty($cartOptions)) {
                                $price_break = DB::table("pricebreaks as pb")->where('pbProdID',$val->cartProdID)->first();
                                if($val->cartProdID == $post['pID'] && $cartOptions->coOptID == $post['optID'] && !empty($price_break)){
                                    if($val->cartQuantity + 1 == $price_break->pbQuantity){
                                        $val->cartProdPrice = $price_break->pPrice;
                                    }
                                }
                                
                                $flag = false;
                                $val->cartQuantity = $val->cartQuantity + 1;
                                if ($val->update()) {
                                    return ['status' => true, 'message' => 'Item added to cart.','data' => $result];
                                } else {
                                    return ['status' => true, 'message' => 'something_wrong'];
                                }
                            }
                        }
                    } else {
                        $price_break = DB::table("pricebreaks as pb")->where('pbProdID',$cart->cartProdID)->first();
                        if(!empty($price_break)){
                            if($cart->cartQuantity + 1 == $price_break->pbQuantity){
                                $cart->cartProdPrice = $price_break->pPrice;
                            }
                        }
                        $cart->cartQuantity = $cart->cartQuantity + 1;
                        
                        if ($cart->update()) {
                            return ['status' => true, 'message' => 'Item added to cart.','data' => $result];
                        } else {
                            return ['status' => true, 'message' => 'something_wrong'];
                        }
                    }
                    
                    if ($flag) {
                        return $this->addProductToCart($post, $product, $user, $option,$tIsGuest,$cartSessionID);
                    }
                }
            } else {
                return ['status' => false, 'message' => 'invalid_product'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    private function addProductToCart($post, $product, $user, $option,$tIsGuest,$cartSessionID) {
        $modelCart = new Cart();
        $modelCart->cartProdID = $post['pID'];
        $modelCart->cartProdName = $product->pName;
        $modelCart->cartProdPrice = $product->pPrice;
        $modelCart->cartDateAdded = date('Y-m-d H:i:s');
        $modelCart->cartQuantity = 1;
        
        if($tIsGuest == 1){
            $modelCart->cartSessionID = $cartSessionID;
        }else{
            $modelCart->cartClientID = $user->clID;
        }
        
        
        $modelCart->cartOrderID = 0;
        $modelCart->cartCompleted = 0;
        if ($modelCart->save()) {
            if (!empty($option) && isset($post['optID']) && trim($post['optID']) != "") {
                $optPrice = $option->optPriceDiff;
                if(trim($option->optRegExp) != ""){
                    $opt = DB::table("products")->select("pPrice")->where("pID",$option->optRegExp)->first();
                    if(!empty($opt)){
                        $optPrice = $opt->pPrice;
                        $modelCart->cartProdID = $option->optRegExp;
                        $modelCart->cartOrigProdID = $post['pID'];
                        $modelCart->cartProdPrice = $optPrice;
                        $modelCart->update();
                    }
                }else{
                    $price_break = DB::table("pricebreaks as pb")->where('pbProdID',$product->pID)->first();
                    if(!empty($price_break)){
                        $optPrice = round(($option->optPriceDiff * $product->pPrice) / 100,2);
                    } else if ($option->optFlags > 0 && $option->optFlags != ""){
                        $optPrice = round(($option->optPriceDiff * $product->pPrice) / 100,2);
                    }
                    
                    $pWeight = (($product->pWeight * $option->optWeightDiff) / 100);
                    if(($option->optFlags & 2) == 0){
                       $pWeight = $option->optWeightDiff;
                    }
                    
                    $cartoptionModel = new Cartoption();
                    $cartoptionModel->coCartID = $modelCart->cartID;
                    $cartoptionModel->coOptID = $post['optID'];
                    $cartoptionModel->coOptGroup = $option->optGrpName;
                    $cartoptionModel->coCartOption = $option->optName;
                    $cartoptionModel->coPriceDiff = $optPrice;
                    $cartoptionModel->coWeightDiff = $pWeight;
                    $cartoptionModel->save();
                }
            }
            
            $result = [
                'cartSessionID' => $cartSessionID
            ];

            return ['status' => true, 'message' => 'Product added to cart.','data' => $result];
        } else {
            return ['status' => false, 'message' => 'something_wrong'];
        }
    }

    public function cart_list($post, $ordID = NULL, $isFedex = false) {
        $response = [];
        try {
            $tIsGuest = 0;
            if(isset($_SERVER['HTTP_TISGUEST']) && trim($_SERVER['HTTP_TISGUEST']) != ""){
                $tIsGuest = trim($_SERVER['HTTP_TISGUEST']);
            }
            
            $user = ($tIsGuest != 1) ? Authentication::getuser() : [];
            
            $cartSessionID = "";
            if($tIsGuest == 1){
                $cartSessionID = trim($_SERVER['HTTP_CARTSESSIONID']);
            }
            
            $data = Cart::from("cart AS c")
                    ->selectRaw("c.cartID,c.cartProdID,c.cartProdName,IF(CONCAT(co.coOptGroup,' - ',co.coCartOption) IS NULL,'',CONCAT(co.coOptGroup,' - ',co.coCartOption)) AS optionName,og.optFlags,o.optWeightDiff,

                            IF(co.coPriceDiff IS NULL OR co.coPriceDiff = 0,c.cartProdPrice,
                                (
                                    CASE WHEN(og.optFlags = 0 OR og.optFlags = 1)
                                    THEN (p.pPrice + co.coPriceDiff)
                                    WHEN(og.optFlags = 3)
                                    THEN (c.cartProdPrice + co.coPriceDiff)
                                    ELSE IF(og.optFlags = 0,(p.pPrice + co.coPriceDiff),(p.pPrice + ((p.pPrice * co.coPriceDiff)/100)))
                                    END
                                )
                            ) AS cartProdPrice,

                            c.cartQuantity,IFNULL(c.cartOrderID,0) AS cartOrderID,p.pExemptions,p.pWeight,co.coWeightDiff,IF(pi.imageSrc IS NULL,'',CONCAT('" . LIVE_URL . "','',pi.imageSrc)) AS imageSrc,(IF(co.coPriceDiff IS NULL OR co.coPriceDiff = 0,c.cartProdPrice,
                                (
                                    CASE WHEN(og.optFlags = 0 OR og.optFlags = 1)
                                    THEN (p.pPrice + co.coPriceDiff)
                                    WHEN(og.optFlags = 3)
                                    THEN (c.cartProdPrice + co.coPriceDiff)
                                    ELSE IF(og.optFlags = 0,(p.pPrice + co.coPriceDiff),(p.pPrice + ((p.pPrice * co.coPriceDiff)/100)))
                                    END
                                )
                            ) * c.cartQuantity) AS dTotalPrice")
                    ->leftjoin('cartoptions AS co', 'co.coCartID', '=', 'c.cartID')
                    ->leftjoin("products AS p", 'p.pID', '=', 'c.cartProdID')
                    ->leftjoin('prodoptions AS po', 'po.poProdID', '=', 'p.pID')
                    ->leftjoin('optiongroup AS og', 'og.optGrpID', '=', 'po.poOptionGroup')
                    ->leftjoin('options AS o', 'o.optGroup', '=', 'og.optGrpID')
                    ->leftjoin("productimages AS pi", "c.cartProdID", "=", "pi.imageProduct");
            
            if ($ordID != NULL) {
                //to get guest checkout cart list
                $data->where('cartOrderID', $ordID);
            } else {
                if($tIsGuest == 1){
                    $data->where('c.cartSessionID', $cartSessionID);
                }else{
                    $data->where('c.cartClientID', $user->clID);
                }
                $data->whereRaw('(c.cartCompleted = 0)')
                        ->whereRaw('(c.cartOrderID = 0)');
            }
            $list = $data->groupBy("c.cartID")->get();
            
            if ($list->count() > 0) {
                $priceArr = array_column($list->toArray(), 'dTotalPrice');
                $dSubTotal = array_sum($priceArr);

                $settings = Common::getAdminSettings();
                $fedex_data = $this->fedexXmlSoapData($settings, $user, $dSubTotal, $list, $post, $tIsGuest, $cartSessionID);
                $shippingCharge = $fedex_data['shipping'];
                $intShipping = $fedex_data['intShipping'];
                
                $fedexOptions = [];
                if(count($intShipping) > 0){
                    foreach ($intShipping as $key => $val){
                        if($val[0]){
                            $fedexOptions[] = [
                                'key' => $val[0],
                                'days' => $val[1],
                                'price' => $val[2]
                            ];
                        }
                    }
                }
                
                $dShippingHandlingCharge = $shippingCharge + $settings->adminHandling;
                $dGrandTotal = $dSubTotal + $dShippingHandlingCharge;
                $dGrandTotalWithoutShipping = $dSubTotal + $settings->adminHandling;
                $result = [
                    'dSubTotal' => $dSubTotal,
                    'dShippingHandlingCharge' => $dShippingHandlingCharge,
                    'dGrandTotal' => $dGrandTotal,
                    'dGrandTotalWithoutShipping' => $dGrandTotalWithoutShipping,
                    'cartList' => $list
                ];
                
                if($isFedex){
                    $result['fedexOptions'] = $fedexOptions;
                }

                $response = ['status' => true, 'message' => 'Cart list', 'data' => $result];
            } else {
                $response = ['status' => true, 'message' => 'no_record'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Fedex xml soap request data
     */

    public function fedexXmlSoapData($settings, $user, $dSubTotal, $list, $post, $tIsGuest, $cartSessionID) {
        $fedexpickuptype = 'REGULAR_PICKUP';
        $fedexpackaging = 'YOUR_PACKAGING';

        /** get user address information * */
        $userAddress = [];
        if (!empty($user)) {
            $userAddress = DB::table("address")
                    ->where('addCustID', $user->clID)
                    ->orderby("addIsDefault", "DESC")
                    ->first();
        }
        
        $countryCode = $settings->countryCode;
        $currencyCode = $settings->countryCurrency;

        if (isset($post['ordCountry']) && trim($post['ordCountry']) != "") {
            if ($post['shipToDifferentAddress'] == 1) {
                $country = DB::table('countries')->where('countryName', $post['ordShipCountry'])->first();
                $countryCode = $country->countryCode;
                $currencyCode = $country->countryCurrency;
            } else {
                $country = DB::table('countries')->where('countryName', $post['ordCountry'])->first();
                $countryCode = $country->countryCode;
                $currencyCode = $country->countryCurrency;
            }
        } else {
            if (isset($post['countryID']) && trim($post['countryID']) != "" && trim($post['countryID']) != 0) {
                $country = DB::table('countries')->where('countryID', $post['countryID'])->first();
                $countryCode = $country->countryCode;
                $currencyCode = $country->countryCurrency;
            } else {
                if (!empty($user)) {
                    if (trim($userAddress->addCountry) != "") {
                        $country = DB::table('countries')->where('countryName', $userAddress->addCountry)->first();
                        $countryCode = $country->countryCode;
                        $currencyCode = $country->countryCurrency;
                    }
                }
            }
        }
        
        $currencyCode = ($currencyCode == 'CAD') ? 'USD' : $currencyCode;
        $addZip = $settings->adminZipCode;

        if (isset($post['ordZip']) && trim($post['ordZip']) != "") {
            if ($post['shipToDifferentAddress'] == 1) {
                $addZip = $post['ordShipZip'];
            }else{
                $addZip = $post['ordZip'];
            }
        } else {
            if (isset($post['zipCode']) && trim($post['zipCode']) != "") {
                $addZip = $post['zipCode'];
            } else {
                if (trim($userAddress->addZip) != "") {
                    $addZip = $userAddress->addZip;
                }
            }
        }
        //get state code from state name of user address
        $stateOrProvinceCode = "";
        $state_name = "";
        
        if(!empty($userAddress) && trim($userAddress->addState) != ""){
            $state_name = $userAddress->addState;
        }
        
        if(isset($post['stateName']) && trim($post['stateName']) != ""){
            $state_name = $post['stateName'];
        }
        
        if(isset($post['shipToDifferentAddress']) && trim($post['shipToDifferentAddress']) == 1){
            $state_name = $post['ordShipState'];
        }else{
            if(isset($post['ordState']) && trim($post['ordState']) != ""){
                $state_name = $post['ordState'];
            }
        }
        
        if(trim($state_name) != "" && $countryCode != "CA"){
            $state_data = DB::table("states")
                ->where('stateName', $state_name)
                ->first();

            if(!empty($state_data)){
                $stateOrProvinceCode = $state_data->stateAbbrev;
            }
        }
        
        $xml = '';
        $xml .= '<soapenv:Envelope
        xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
        xmlns:v9="http://fedex.com/ws/rate/v9">
        <soapenv:Header/>
        <soapenv:Body>
        <v9:RateRequest>
            <v9:WebAuthenticationDetail>
                <v9:CspCredential>
                    <v9:Key>mKOUqSP4CS0vxaku</v9:Key>
                    <v9:Password>IAA5db3Pmhg3lyWW6naMh4Ss2</v9:Password>
                </v9:CspCredential>
                <v9:UserCredential>
                    <v9:Key>' . $settings->FedexUserKey . '</v9:Key>
                    <v9:Password>' . $settings->FedexUserPwd . '</v9:Password>
                </v9:UserCredential>
            </v9:WebAuthenticationDetail>
            <v9:ClientDetail>
                <v9:AccountNumber>' . $settings->FedexAccountNo . '</v9:AccountNumber>
                <v9:MeterNumber>' . $settings->FedexMeter . '</v9:MeterNumber>
                <v9:ClientProductId>IBTP</v9:ClientProductId>
                <v9:ClientProductVersion>3272</v9:ClientProductVersion>
            </v9:ClientDetail>
            <v9:TransactionDetail>
                <v9:CustomerTransactionId>Rate Request</v9:CustomerTransactionId>
            </v9:TransactionDetail>
            <v9:Version>
                <v9:ServiceId>crs</v9:ServiceId>
                <v9:Major>9</v9:Major>
                <v9:Intermediate>0</v9:Intermediate>
                <v9:Minor>0</v9:Minor>
            </v9:Version>
            <v9:ReturnTransitAndCommit>true</v9:ReturnTransitAndCommit>
            <v9:RequestedShipment>
                <v9:DropoffType>' . $fedexpickuptype . '</v9:DropoffType>
                <v9:PackagingType>' . $fedexpackaging . '</v9:PackagingType>
                <v9:Shipper>
                    <v9:Address>
                        <v9:PostalCode>' . $settings->adminZipCode . '</v9:PostalCode>
                        <v9:CountryCode>' . $settings->countryCode . '</v9:CountryCode>
                    </v9:Address>
                </v9:Shipper>
                <v9:Recipient>
                    <v9:Address>
                        <v9:StateOrProvinceCode>'.$stateOrProvinceCode.'</v9:StateOrProvinceCode>
                        <v9:PostalCode>' . $addZip . '</v9:PostalCode>
                        <v9:CountryCode>' . $countryCode . '</v9:CountryCode>
                        <v9:Residential>true</v9:Residential>
                    </v9:Address>
                </v9:Recipient>
                <v9:SpecialServicesRequested></v9:SpecialServicesRequested>
                <v9:CustomsClearanceDetail>
                    <v9:CustomsValue>
                        <v9:Currency>' . $settings->countryCurrency . '</v9:Currency>
                        <v9:Amount>' . $dSubTotal . '</v9:Amount>
                    </v9:CustomsValue>
                </v9:CustomsClearanceDetail>
                <v9:RateRequestTypes>ACCOUNT</v9:RateRequestTypes>
                <v9:PackageDetail>INDIVIDUAL_PACKAGES</v9:PackageDetail>';
                
        foreach ($list as $key => $val) {
            $pExemptions = 0;
            if ($val->pWeight <= 0) {
                $pExemptions = 4;
            }
            if ($pExemptions == 0) {
                $numpacks = $this->splitlargepacks($val['cartQuantity'], $settings, ($val->pWeight + $val->coWeightDiff));
                $all_qty = $val['cartQuantity'];
                if ($numpacks > 0 && $numpacks != "" && $numpacks != NULL) {
                    $all_qty = $numpacks * $val['cartQuantity'];
                }
                
                for ($i = 0; $i < $all_qty; $i++) {
                    if((($val->pWeight + $val->coWeightDiff) / $numpacks) > 0){
                        $xml .= '<v9:RequestedPackageLineItems>
                                <v9:InsuredValue>
                                    <v9:Currency>' . $currencyCode . '</v9:Currency>
                                    <v9:Amount>' . number_format((float)$val->cartProdPrice, 2, '.', '') . '</v9:Amount>
                                </v9:InsuredValue>
                                <v9:Weight>
                                    <v9:Units>' . ($settings->adminUnits == 1 ? 'LB' : 'KG') . '</v9:Units>
                                    <v9:Value>' . number_format((float)(($val->pWeight + $val->coWeightDiff) / $numpacks), 1, '.', '') . '</v9:Value>
                                </v9:Weight>
                                <v9:SpecialServicesRequested></v9:SpecialServicesRequested>
                            </v9:RequestedPackageLineItems>';
                    }
                }
            }
        }

        $xml .= '</v9:RequestedShipment>
        </v9:RateRequest>
    </soapenv:Body>
</soapenv:Envelope>';

        $cfres = $fedexnamespace = NULL;
        $cferrmsg = '';
        
        $shippingInfo = Common::callcurlfunction(FEDEX_URl, $xml, $cfres, '', $cferrmsg, FALSE);
        
        if ($shippingInfo) {
            $this->dumpxmloutput($xml, $cfres);

            $pattern = '/<(.{1,3}):RateReply/';
            if (preg_match($pattern, $cfres, $matches)) {
                $fedexnamespace = $matches[1] . ':';
            } else {
                $fedexnamespace = '';
            }

            $intShipping = [];
            for ($i = 0; $i < MAX_SHIP_OPTIONS; $i++) {
                $intShipping[$i][0] = ''; // Name
                $intShipping[$i][1] = ''; // Delivery
                $intShipping[$i][2] = 0; // Cost
                $intShipping[$i][3] = 0; // Used
                $intShipping[$i][4] = 0; // FSA
                $intShipping[$i][5] = ''; // Service ID (USPS)
                $intShipping[$i][6] = $settings->adminShipping; // shipType
                $intShipping[$i][7] = 0; // Cost for Free Ship Exempt
            }

            $_param = [
                'xxDay' => 'day',
                'xxDays' => 'days',
                'uselistshippingrates' => 1,
                'commercialloc_' => '',
                'origCountryCode' => $settings->countryCode,
                'shipCountryCode' => $countryCode,
                'nofedexinternationalground' => true,
                'fedextestmode' => null,
                'fedexnamespace' => '',
                'discountshippingfedex' => null,
            ];
            
            $success = Common::parsefedexXMLoutput($cfres, '', '', '', $intShipping, $_param);
            $shipping = 0;
            if ($success) {
                $sortShippingData = Common::sortshippingarray($intShipping, MAX_SHIP_OPTIONS);
                $intShipping = $sortShippingData['intShipping'];
                for ($i = 0; $i < $sortShippingData['maxshipoptions']; $i++) {
                    if ($intShipping[$i][3] == TRUE) {
                        if (!$gotshipping || ($intShipping[$i][4] && $freeshippingapplied)) {
                            $shipping = $intShipping[$i][2];
                            $shipMethod = $intShipping[$i][0];
                            $selectedshiptype = $i;
                            $gotshipping = TRUE;
                        }
                        if ($intShipping[$i][4] && $freeshippingapplied)
                            $freeshipamnt = $intShipping[$i][2] - $intShipping[$i][7];
                    }
                }
            }
            return ['shipping' => $shipping,'intShipping' => $intShipping];
        }
    }

    public function dumpxmloutput($xml, $cfres) {
        print $sentxml;
        print $recvdxml;
//        print str_replace('<','',str_replace('</','&lt;/',$sentxml)) . "\n";
//	print str_replace('<','&lt;',str_replace('</','&lt;/',$recvdxml)) . "\n";
    }

    public function splitlargepacks($cartQuantity, $settings, $thepweight) {
        //global $packdims, $shipType, $adminUnits, $nosplitlargepacks, $thepweight, $splitpackat, $adminCanPostLogin;
        $packtogether = FALSE;
        $adminUnits = 1;
        $shipType = $settings->adminShipping;
        $nosplitlargepacks = NULL;
        $splitpackat = NULL;
        $adminCanPostLogin = '';

        $packdims = $this->addpackagedimensions('', ($packtogether ? (int) $cartQuantity : 1), $adminUnits);

        $slpweight = $thepweight;
        $slpnumpacks = 1;
        if ($packdims[7] <= 1 && @$nosplitlargepacks == TRUE)
            return 1;
        if ($shipType == 6) {
            if (($adminUnits & 12) == 4) {
                $maxlenplusgirth = 118;
                $maxlength = 78;
            } else {
                $maxlenplusgirth = 300;
                $maxlength = 200;
            }
            if (($adminUnits & 3) == 1)
                $maxweight = 66;
            else
                $maxweight = 30;
            if ($adminCanPostLogin != '')
                $nosplitlargepacks = TRUE;
        }elseif ($shipType == 4 || $shipType == 7 || $shipType == 8 || $shipType == 9) {
            if (($adminUnits & 12) == 4) {
                $maxlenplusgirth = 165;
                $maxlength = 108;
            } else {
                $maxlenplusgirth = 419;
                $maxlength = 274;
            }
            if (($adminUnits & 3) == 1)
                $maxweight = 150;
            else
                $maxweight = 68;
        }else { // USPS Default
            if (($adminUnits & 12) == 8)
                $maxlenplusgirth = 330;
            else
                $maxlenplusgirth = 130;
            $maxlength = 0;
            $maxweight = 70;
        }
        if (is_numeric(@$splitpackat))
            $maxweight = (double) $splitpackat;
        if (@$nosplitlargepacks != TRUE && ($adminUnits & 12) != 0) {
            if ($packdims[0] + (($packdims[1] + $packdims[2]) * 2) > $maxlenplusgirth) { // Max Length + Girth
                $divisor = 1;
                while (($packdims[0] / sqrt($divisor)) + ((($packdims[1] / sqrt($divisor)) + $packdims[2]) * 2) > $maxlenplusgirth)
                    $divisor++;
                if ($packdims[0] / sqrt($divisor) > $maxlength && $maxlength != 0 && ($packdims[0] / $divisor) + (($packdims[1] + $packdims[2]) * 2) <= $maxlenplusgirth)
                    $packdims[0] = $packdims[0] / $divisor;
                else {
                    $packdims[0] = $packdims[0] / sqrt($divisor);
                    $packdims[1] = $packdims[1] / sqrt($divisor);
                }
                $slpnumpacks *= $divisor;
                $slpweight /= $divisor;
                $this->reorderpackagedimensions();
            }
            if ($packdims[0] > $maxlength && $maxlength != 0) {
                $packdims[0] = $packdims[0] / 2;
                $slpnumpacks *= 2;
                $slpweight /= 2;
                $this->reorderpackagedimensions();
            }
        }
        if ($slpweight > $maxweight && ($packdims[7] <= $slpnumpacks || @$nosplitlargepacks != TRUE)) {
            $slpindex = 2;
            while (TRUE) {
                if ($slpweight / $slpindex <= $maxweight) {
                    $packdims[0]/=$slpindex;
                    $slpnumpacks*=$slpindex;
                    $this->reorderpackagedimensions();
                    break;
                }
                $slpindex++;
            }
        }
        return($slpnumpacks);
    }

    private function reorderpackagedimensions() {
        global $packdims;
        if ($packdims[2] > $packdims[1]) {
            $apdtemp = $packdims[1];
            $packdims[1] = $packdims[2];
            $packdims[2] = $apdtemp;
        }
        if ($packdims[1] > $packdims[0]) {
            $apdtemp = $packdims[0];
            $packdims[0] = $packdims[1];
            $packdims[1] = $apdtemp;
        }
        if ($packdims[2] > $packdims[1]) {
            $apdtemp = $packdims[1];
            $packdims[1] = $packdims[2];
            $packdims[2] = $apdtemp;
        }
    }

    private function addpackagedimensions($dimens, $apdquant, $adminUnits) {
        $packdims = array(0, 0, 0, 0, 0, 0, 0, 0);

        if (($adminUnits & 12) != 0) {
            $origdimens = $packdims;
            $proddims = explode('x', $dimens);
            if (@$proddims[0] != '' && @$proddims[1] != '' && @$proddims[2] != '') {
                $this->reorderproddims($proddims);
                if ($proddims[0] > $packdims[4])
                    $packdims[4] = $proddims[0];
                if ($proddims[1] > $packdims[5])
                    $packdims[5] = $proddims[1];
                if ($proddims[2] > $packdims[6])
                    $packdims[6] = $proddims[2];
                $proddims[2] = $proddims[2] * $apdquant;
                $thelength = $proddims[0];
                $this->reorderproddims($proddims);
                while ($apdquant > 4 && $proddims[0] > $proddims[2] * 2 && $proddims[0] > $thelength) {
                    $proddims[0] = $proddims[0] / 2;
                    $proddims[2] = $proddims[2] * 2;
                    $apdquant = $apdquant / 2;
                    $this->reorderproddims($proddims);
                }
                $thelength = $proddims[0];
                $thewidth = $proddims[1];
                $theheight = $proddims[2];
                $objvol = $thelength * $thewidth * $theheight;
                if ($thelength > $packdims[0])
                    $packdims[0] = $thelength;
                if ($thewidth > $packdims[1])
                    $packdims[1] = $thewidth;
                if ($theheight > $packdims[2])
                    $packdims[2] = $theheight;
                if ($objvol + $packdims[3] > $packdims[0] * $packdims[1] * $packdims[2])
                    $packdims[2] = $packdims[2] + ($origdimens[2] > 0 && $origdimens[2] < $theheight ? $origdimens[2] : $theheight);
                if ($objvol + $packdims[3] > $packdims[0] * $packdims[1] * $packdims[2])
                    $packdims[1] = $packdims[1] + ($origdimens[1] > 0 && $origdimens[1] < $thewidth ? $origdimens[1] : $thewidth);
                if ($objvol + $packdims[3] > $packdims[0] * $packdims[1] * $packdims[2])
                    $packdims[0] = $packdims[0] + ($origdimens[0] > 0 && $origdimens[0] < $thelength ? $origdimens[0] : $thelength);
                $packdims[3] = $packdims[3] + $objvol;
                reorderpackagedimensions();
            }
        }
        $packdims[7]+=$apdquant;
        return $packdims;
        // print "Bin is : " . $packdims[0] . ':' . $packdims[1] . ':' . $packdims[2] . '=' . ($packdims[0]*$packdims[1]*$packdims[2]) . '<br />';
    }

    private function reorderproddims(&$pdims) {
        $pdims[0] = (double) $pdims[0];
        $pdims[1] = (double) $pdims[1];
        $pdims[2] = (double) $pdims[2];
        if ($pdims[2] > $pdims[1]) {
            $apdtemp = $pdims[1];
            $pdims[1] = $pdims[2];
            $pdims[2] = $apdtemp;
        }
        if ($pdims[1] > $pdims[0]) {
            $apdtemp = $pdims[0];
            $pdims[0] = $pdims[1];
            $pdims[1] = $apdtemp;
        }
        if ($pdims[2] > $pdims[1]) {
            $apdtemp = $pdims[1];
            $pdims[1] = $pdims[2];
            $pdims[2] = $apdtemp;
        }
    }

    /*
     * Remove product from cart
     */

    public function removeCartProduct($post,$tIsGuest) {
        try {
            $user = ($tIsGuest != 1) ? Authentication::getuser() : [];
            
            $cartSessionID = "";
            if($tIsGuest == 1){
                $cartSessionID = trim($_SERVER['HTTP_CARTSESSIONID']);
            }
            
            
            $list = Cart::where('cartID',$post['cartID']);
            if($tIsGuest == 1){
                $list->where('cartSessionID',$cartSessionID);
            }else{
                $list->where('cartClientID', $user->clID);
            }
                    
            $cart = $list->first();
            if (!empty($cart)) {
                $isDeleted = $cart->delete();
                if ($isDeleted) {
                    DB::table('cartoptions')->where(['coCartID' => $post['cartID']])->delete();
                    return ['status' => true, 'message' => 'Item removed from cart.'];
                } else {
                    return ['status' => false, 'message' => 'cart_remove_fail'];
                }
            } else {
                return ['status' => false, 'message' => 'invalid_cart_item'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    public function updateCartQty($post,$tIsGuest) {
        try {
            $user = ($tIsGuest != 1) ? Authentication::getuser() : [];
            
            $cartSessionID = "";
            if($tIsGuest == 1){
                $cartSessionID = trim($_SERVER['HTTP_CARTSESSIONID']);
            }
            
            $list = Cart::where('cartID',$post['cartID']);
            if($tIsGuest == 1){
                $list->where('cartSessionID', $cartSessionID);
            }else{
                $list->where('cartClientID', $user->clID);
            }
            
            $cart = $list->first();
            
            if (!empty($cart)) {
                $flag = false;
                if (trim($post['cartQuantity']) > 0) {
                    $price_break = DB::table("pricebreaks as pb")->where('pbProdID',$cart->cartProdID)->first();
                    if(!empty($price_break)){
                        if($post['cartQuantity'] >= $price_break->pbQuantity){
                            $cart->cartProdPrice = $price_break->pPrice;
                        }else{
                            $cart->cartProdPrice = $cart->product->pPrice;
                        }
                    }
                    
                    $cart->cartQuantity = $post['cartQuantity'];
                    if ($cart->update()) {
                        $flag = true;
                    } else {
                        return ['status' => false, 'message' => 'cart_update_fail'];
                    }
                } else {
                    $isDeleted = $cart->delete();
                    if ($isDeleted) {
                        DB::table('cartoptions')->where(['coCartID' => $post['cartID']])->delete();
                        $flag = true;
                    } else {
                        return ['status' => false, 'message' => 'cart_remove_fail'];
                    }
                }

                if ($flag) {
                    $params = [];
                    $cart = $this->cart_list($params);

                    if ($cart['status']) {
                        $result = ['dSubTotal' => $cart['data']['dSubTotal'], 'dShippingHandlingCharge' => $cart['data']['dShippingHandlingCharge'], 'dGrandTotal' => $cart['data']['dGrandTotal']];
                        return ['status' => true, 'message' => 'cart_qty_updated', 'data' => $result];
                    }
                }
            } else {
                return ['status' => false, 'message' => 'invalid_cart_item'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Get guest cart shipping charge
     */

    public function getGuestCartShippingCharge($post) {
        $cart_json = json_decode($post['cart_json'], true);

        if (!empty($cart_json)) {
            $list = [];
            foreach ($cart_json as $key => $val) {
                if (trim($val['coOptID']) != "" && $val['coOptID'] != 0) {
                    $list[] = Option::from('options AS o')
                            ->selectRaw($val['cartQuantity'] . " as cartQuantity,p.pExemptions,p.pWeight,o.optWeightDiff AS coWeightDiff,
                                (
                                    CASE WHEN(og.optFlags = 0)
                                    THEN ROUND((p.pPrice + o.optPriceDiff),1)
                                    ELSE ROUND(IF(og.optFlags = 0,(p.pPrice + o.optPriceDiff),(p.pPrice + ((p.pPrice * o.optPriceDiff)/100))))
                                    END
                                ) AS cartProdPrice,
                                
                                ((
                                    CASE WHEN(og.optFlags = 0)
                                    THEN ROUND((p.pPrice + o.optPriceDiff),1)
                                    ELSE ROUND(IF(og.optFlags = 0,(p.pPrice + o.optPriceDiff),(p.pPrice + ((p.pPrice * o.optPriceDiff)/100))))
                                    END
                                ) * ".$val['cartQuantity'].") AS dTotalPrice

                                    ")
                            ->leftjoin('optiongroup AS og', 'og.optGrpID', '=', 'o.optGroup')
                            ->leftjoin('prodoptions as po', 'po.poOptionGroup', '=', 'og.optGrpID')
                            ->leftjoin('products as p', 'p.pID', '=', 'po.poProdID')
                            ->where(['p.pID' => $val['cartProdID'],'o.optID' => $val['coOptID']])
                            ->first();
                 
                    
                    
                } else {
                    $list[] = Product::from("products AS p")
                            ->selectRaw("p.pPrice as cartProdPrice," . $val['cartQuantity'] . " as cartQuantity,(p.pPrice * " . $val['cartQuantity'] . ") AS dTotalPrice,p.pExemptions,p.pWeight")
                            ->where('p.pID', $val['cartProdID'])
                            ->first();
                }
            }
            
            if (!empty($list)) {
                $priceArr = array_column($list, 'dTotalPrice');
                $dSubTotal = array_sum($priceArr);

                $settings = Common::getAdminSettings();
                $shippingCharge = $this->fedexXmlSoapData($settings, [], $dSubTotal, $list, $post);
                $dShippingHandlingCharge = $shippingCharge + $settings->adminHandling;
                $dGrandTotal = $dSubTotal + $dShippingHandlingCharge;

                $result = [
                    'dSubTotal' => $dSubTotal,
                    'dShippingHandlingCharge' => $dShippingHandlingCharge,
                    'dGrandTotal' => $dGrandTotal,
                ];

                $response = ['status' => true, 'message' => 'Cart list', 'data' => $result];
            }
        }
        return $response;
    }

}
