<?php

use Illuminate\Database\Eloquent\Model as Eloquent;
use Illuminate\Database\Capsule\Manager as DB;

class Customerlogin extends Eloquent {

    protected $table = 'customerlogin';
    protected $primaryKey = 'clID';
    public $timestamps = false;
    
    public function device() {
        return $this->hasOne(Device::class, 'clID', 'clID');
    }
    
    public function login($post) {
        $response = [];
        DB::beginTransaction();
        try {
            $user = Customerlogin::where(['clEmail' => $post['clEmail'], 'clPW' => md5(MD5_HASH_KEY . $post['clPW'])])
                    ->first();
            //validation use exist or not
            if (!empty($user)) {
                $device = Device::where(['clID' => $user->clID])->first();
                //Updating device information
                if (!empty($device)) {
                    $device->txDeviceToken = $post['txDeviceToken'];
                    $device->tDeviceOs = $post['tDeviceOs'];
                    $device->vDeviceName = (isset($post['vDeviceName']) && trim($post['vDeviceName']) != "") ? $post['vDeviceName'] : '';
                    $device->vResolution = (isset($post['vResolution']) && trim($post['vResolution']) != "") ? $post['vResolution'] : '';
                    $device->vOsVersion = (isset($post['vOsVersion']) && trim($post['vOsVersion']) != "") ? $post['vOsVersion'] : '';
                    $device->save();
                }
                $result = $this->userResponse($user);
                $response = ['status' => true, 'message' => 'login_success', 'data' => $result];
                DB::commit();
            } else {
                $response = ['status' => false, 'message' => 'wrong_email_password'];
            }
        } catch (Exception $ex) {
            DB::rollback();
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Signup Customer
     */

    public function signup($post) {
        $response = [];
        DB::beginTransaction();
        try {
            //Check email already exist or not
            if ($this->isEmailRegistered($post['clEmail'])) {
               
                    //Create customer
                    $model = new Customerlogin();
                    $model->clUserName = $post['clUserName'];
                    $model->clPW = md5(MD5_HASH_KEY . $post['clPW']);
                    $model->clEmail = $post['clEmail'];
                    $model->clDateCreated = date('Y-m-d H:i:s');
                    $model->save();

                    //Add device entry
                    $deviceModel = new Device();
                    $deviceModel->clID = $model->clID;
                    $deviceModel->txDeviceToken = $post['txDeviceToken'];
                    $deviceModel->tDeviceOs = $post['tDeviceOs'];
                    $deviceModel->vDeviceName = (isset($post['vDeviceName']) && trim($post['vDeviceName']) != "") ? $post['vDeviceName'] : '';
                    $deviceModel->vResolution = (isset($post['vResolution']) && trim($post['vResolution']) != "") ? $post['vResolution'] : '';
                    $deviceModel->vOsVersion = (isset($post['vOsVersion']) && trim($post['vOsVersion']) != "") ? $post['vOsVersion'] : '';
                    $deviceModel->updated_at = date('Y-m-d H:i:s');
                    $deviceModel->created_at = date('Y-m-d H:i:s');
                    $deviceModel->save();
                    $result = $this->userResponse($model);

                    DB::commit();
                    $response = ['status' => true, 'message' => 'signup_success', 'data' => $result];
                
            } else {
                DB::rollback();
                $response = ['status' => false, 'message' => 'email_already_exist'];
            }
        } catch (Exception $ex) {
            DB::rollback();
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Email already exist check
     */

    private function isEmailRegistered($clEmail) {
        $user = Customerlogin::selectRaw("count(*) AS count")->where(['clEmail' => $clEmail])
                ->first();
        if (empty($user) || $user->count == 0) {
            return true;
        } else {
            return false;
        }
    }

    /*
     * Build user common response
     */

    private function userResponse($model) {
        if (!empty($model)) {
            return [
                'clID' => $model->clID,
                'clUserName' => $model->clUserName,
                'clEmail' => $model->clEmail,
            ];
        } else {
            return (object) [];
        }
    }

    /*
     * Forgot password mail send
     */

    public function forgotpassword($post) {
        $response = [];
        try {
            $user = Customerlogin::where(['clEmail' => $post['clEmail']])
                    ->first();
            //Validate email exist or not
            if (!empty($user)) {
                $params = [
                    'clEmail' => $user->clEmail,
                    'clPW' => $user->clPW
                ];

                $to = $post['clEmail'];
                $from = SMTP_EMAIL;
                $from_name = SITENAME;
                $subject = APP_NAME . ' | Forgot password';
                $body = Common::mailTemplate('forgotpassword', $params);

                $mailRes = Common::send_mail($to, $from, $from_name, $subject, $body);
                if ($mailRes) {
                    $response = ['status' => true, 'message' => 'Pasword reset mail sent to your email address, Please check your mail inbox.'];
                } else {
                    $response = ['status' => false, 'message' => 'mail_sending_fail'];
                }
            } else {
                $response = ['status' => false, 'message' => 'invalid_email'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Change user password
     */

    public function change_password($post) {
        $response = [];
        try {
            $user = Authentication::getuser();
            if ($user->clPW == md5(MD5_HASH_KEY . $post['vOldPassword'])) {
                $user->clPW = md5(MD5_HASH_KEY . $post['vNewPassword']);
                $user->save();
                $response = ['status' => true, 'message' => 'password_changed'];
            } else {
                $response = ['status' => false, 'message' => 'old_password_wrong'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Country list
     */

    public function country_list() {
        $response = [];
        try {
            $result = DB::table('countries')
                    ->select('countryID', 'countryName')
                    ->where('countryEnabled', 1)
                    ->orderBy('countryID')
                    ->get();
            $response = ['status' => true, 'message' => 'Country list', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * State list for custom quote request
     */

    public function customQuoteOptions() {
        $response = [];
        try {
            /*             * *** State lsit *** */
            $stateArr = [
                'Alabama', 'Alaska', 'Arizona', 'Arkansas', 'California', 'Colorado', 'Connecticut', 'Delaware', 'Florida', 'Georgia', 'Hawaii', 'Idaho', 'Illinois', 'Indiana', 'Iowa', 'Kansas', 'Kentucky', 'Louisiana', 'Maine', 'Maryland', 'Massachusetts', 'Michigan', 'Minnesota', 'Mississippi', 'Missouri', 'Montana', 'Nebraska', 'Nevada', 'New Hampshire', 'New Jersey', 'New Mexico', 'New York', 'North Carolina', 'North Dakota', 'Ohio', 'Oklahoma', 'Oregon', 'Pennsylvania', 'Rhode Island', 'South Carolina', 'South Dakota', 'Tennessee', 'Texas', 'Utah', 'Vermont', 'Virginia', 'Washington', 'West Virginia', 'Wisconsin', 'Wyoming', 'CANADA', 'Alberta', 'British Columbia', 'Labrador', 'Manitoba', 'New Brunswick', 'Newfoundland', 'North West Terr', 'Nova Scotia', 'Nunavut', 'Ontario', 'Prince Edward Island', 'Quebec', 'Saskatchewen', 'Yukon', 'Other State'
            ];

            $state_list = [];
            foreach ($stateArr as $key => $val) {
                $state_list[] = [
                    'vName' => $val
                ];
            }

            /*             * * Custome made ** */
            $customMadeArr = [
                'VCI Paper-Ferrous', 'VCI Paper-Multi-metal', 'Alumitex', 'Coppertex', 'Silver Saver', 'VCI Poly', 'VCI Poly Bags Covers', 'Silver Guard Poly', 'Foil Bags/Covers', 'Foil Roll Stock', 'Other'
            ];
            $custom_made_list = [];
            foreach ($customMadeArr as $key => $val) {
                $custom_made_list[] = [
                    'vName' => $val
                ];
            }

            /*             * * Thickness ** */
            $thicjnessArr = [
                '2 Mil', '3 Mil', '4 Mil', '5 Mil', '6 Mil', '7 Mil', '8 Mil'
            ];
            $thickness_list = [];
            foreach ($thicjnessArr as $key => $val) {
                $thickness_list[] = [
                    'vName' => $val
                ];
            }

            /*             * * Purchase Quentity ** */
            $purchaseQtyArr = [
                '250', '500', '1000', '2000', '3000', '4000', '5000', '10000', '15000', '20000'
            ];
            $purchase_qty_list = [];
            foreach ($purchaseQtyArr as $key => $val) {
                $purchase_qty_list[] = [
                    'vName' => $val
                ];
            }
            //Purchase frequency
            $purchase_frequency_list = [
                ['vName' => 'Weekly'],
                ['vName' => 'Monthly'],
                ['vName' => 'Quarterly'],
                ['vName' => 'Annual'],
            ];

            //Type of product list (for vci selection help)
            $type_of_product_want_to_protect = [
                ['vName' => 'Ferrous metals'], ['vName' => 'Cast iron'], ['vName' => 'steel'], ['vName' => 'stainless steel'], ['vName' => 'Copper'], ['vName' => 'Silver'], ['vName' => 'Aluminum'], ['vName' => 'Brass'], ['vName' => 'Galvanized metals'], ['vName' => 'Mixed Ferrous & Nonferrous metals']
            ];

            //Is this for domestic or export of the items? (for vci selection help)
            $for_domestic_or_export = [
                ['vName' => 'Domestic'], ['vName' => 'Export'], ['vName' => 'Long Term Storage']
            ];

            //What is your preferred method of corrosion protection?
            $preferred_method_of_corrosion_protection = [
                ['vName' => 'I don\'t know'], ['vName' => 'VCI Poly Bags'], ['vName' => 'VCI Poly Sheeting'], ['vName' => 'VCI Poly Shrouds'], ['vName' => 'VCI Poly Heat Shrink'], ['vName' => 'Silver Guard Poly Bags'], ['vName' => 'VCI Emitters'], ['vName' => 'VCI Paper'], ['vName' => 'VCI Powder'], ['vName' => 'Hydrotesting Additive'], ['vName' => 'Rinse Additive'], ['vName' => 'Oil Additive'], ['vName' => 'Long Term Lay Up']
            ];

            //SearchType for product search screen dropdown
            $search_type = [
                [
                    'vKey' => 1,
                    'vName' => 'All words'
                ],
                [
                    'vKey' => 2,
                    'vName' => 'Any words'
                ],
                [
                    'vKey' => 3,
                    'vName' => 'Exact Phrase'
                ]
            ];

            $result = [
                'stateList' => $state_list,
                'customMadeList' => $custom_made_list,
                'thicknessList' => $thickness_list,
                'purchaseQtyList' => $purchase_qty_list,
                'purchaseFrequencyList' => $purchase_frequency_list,
                'typeOfProductWantToProtect' => $type_of_product_want_to_protect,
                'forDomesticOrExport' => $for_domestic_or_export,
                'preferredMethodOfCorrosionProtection' => $preferred_method_of_corrosion_protection,
                'searchType' => $search_type
            ];

            $response = ['status' => true, 'message' => 'Custom quote options list', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Generate client token
     */

    public function generateClientToken($post) {
        try {
            
            $tIsGuest = 0;
            if(isset($_SERVER['HTTP_TISGUEST']) && trim($_SERVER['HTTP_TISGUEST']) != ""){
                $tIsGuest = trim($_SERVER['HTTP_TISGUEST']);
            }
            
            $user = ($tIsGuest != 1) ? Authentication::getuser() : [];
            
            $cartSessionID = "";
            if($tIsGuest == 1){
                $cartSessionID = trim($_SERVER['HTTP_CARTSESSIONID']);
            }
            
            $cartObj = new Cart();
            $cart_response = $cartObj->cart_list($post,NULL,true);
            
            $gateway = Common::braintreeObject();
            $clientToken = $gateway->clientToken()->generate();
            $settings = Common::getAdminSettings();
            
            $result = [
                'clientToken' => $clientToken,
                'ordHandling' => (!empty($settings)) ? $settings->adminHandling : 0,
                'fedexOptions' => $cart_response['data']['fedexOptions']
            ];
            return ['status' => true, 'message' => 'operation_success', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
    }
    
    /*
     * Ask a question
     */
    public function askQuestion($post){
        $product = Product::from('products AS p')
                    ->selectRaw('p.pID,p.pName')
                    ->where('p.pID', $post['pID'])
                    ->first();
        
        $post['pName'] = (!empty($product)) ? $product->pName : "";
        $post['link'] = (!empty($product)) ? LIVE_URL.'proddetail.php?prod='.$post['pID'] : "";
            
        $to = SMTP_EMAIL;
        $from = $post['vEmail'];
        $from_name = $post['vName'];
        $subject = 'Ask a question at our store';
        $body = Common::mailTemplate('ask_question', $post);

        $mailRes = Common::send_mail($to, $from, $from_name, $subject, $body);
        if ($mailRes) {
             return ['status' => true, 'message' => 'ask_question_mail_sent'];
        } else {
            return ['status' => true, 'message' => 'something_wrong'];
        }
    }
}
