<?php

use Illuminate\Database\Eloquent\Model as Eloquent;
use Illuminate\Database\Capsule\Manager as DB;

class Notification extends Eloquent {

    protected $table = 'notifications';
    protected $primaryKey = 'iNotificationId';
    public $timestamps = false;
    
    public function receiver_user() {
        return $this->hasOne(Customerlogin::class, 'clID', 'iReceiverUserId');
    }

    public function sender_user() {
        return $this->hasOne(Customerlogin::class, 'clID', 'iSenderUserId');
    }
    
    public function notificationList(){
        $response = [];
        try {
            $user = Authentication::getuser();

            $list = Notification::where(['iReceiverUserId' => $user->clID]);
            $list->orderBy("created_at","DESC");
            //Pagination
            $totalRecord = $list->get()->count();
            $reqPageno = (isset($post['pageno']) && trim($post['pageno']) != "") ? $post['pageno'] : 1;
            $recordPerPage = (isset($post['count'])) ? $post['count'] : RECORD_PER_PAGE;
            $recPages = ($totalRecord == 0) ? 0 : ($totalRecord < $recordPerPage) ? 1 : $totalRecord / $recordPerPage;
            $pageno = ($reqPageno - 1) * $recordPerPage;
            $nextPage = ($reqPageno < ceil($recPages)) ? $reqPageno + 1 : 0;

            $list->offset($pageno)->limit($recordPerPage);
            $notificationList = $list->get();
            

            $result = [];

            if (!empty($notificationList)) {
                foreach ($notificationList as $key => $val) {
                    $param = json_decode($val->txParameters, true);
                    $result[$key]['iNotificationId'] = $val->iNotificationId;
                    $result[$key]['tNotificationType'] = $val->tNotificationType;
                    $result[$key]['created_at'] = (trim($val->created_at) != "") ? Common::timeElapsedString($val->created_at) : '';
                    $result[$key]['tRead'] = $val->tRead;
                    $result[$key]['txParameters'] = json_decode($val->txParameters, true);
                    $result[$key]['vTitle'] = $val->vTitle;

                    if ($val->tNotificationType == 1) {
                        //Product purchse
                        
                        $result[$key]['vText'] = 'Your order is placed successfully with order id '.$param['ordID'];
                    } else {
                        $result[$key]['vText'] = $val->vText;
                    }
                }
            }
            
            return [
                'status' => true, 
                'message' => 'Notification list',
                'totalRecord' => $totalRecord,
                'totalPage' => ceil($recPages),
                'nextPage' => $nextPage,
                'data' => $result
            ];
        } catch (\Exception $ex) {
            return ['status' => false, 'result' => ['message' => Lang::get('message.something_wrong')]];
        }
    }
}