<?php

use Illuminate\Database\Eloquent\Model as Eloquent;
use Illuminate\Database\Capsule\Manager as DB;

class Order extends Eloquent {

    protected $table = 'orders';
    protected $primaryKey = 'ordID';
    public $timestamps = false;

    /*
     * Order Checkout api
     */

    public function checkout($post,$tIsGuest) {
        DB::beginTransaction();
        try {
            
            $cartSessionID = "";
            if($tIsGuest == 1){
                $cartSessionID = trim($_SERVER['HTTP_CARTSESSIONID']);
            }
            
            $cartObj = new Cart();
            $cart_response = $cartObj->cart_list($post);
            
            $user = ($tIsGuest != 1) ? Authentication::getuser() : [];
            
            $model = new Order();
            $model->ordClientID = ($tIsGuest != 1) ? $user->clID : 0;
            $model->ordSessionID = ($tIsGuest == 1) ? $cartSessionID : NULL;
            $model->ordExtra1 = $post['ordCompany'];
            $model->ordCountry = $post['ordCountry'];
            $model->ordName = $post['ordName'];
            $model->ordAddress = $post['ordAddress'];
            $model->ordCity = $post['ordCity'];
            $model->ordState = $post['ordState'];
            $model->ordZip = $post['ordZip'];
            $model->ordPhone = $post['ordPhone'];
            $model->ordEmail = $post['ordEmail'];
            $model->ordTransID = (isset($post['ordTransID']) && trim($post['ordTransID']) != "") ? $post['ordTransID'] : '';
            $model->ordPayProvider = $post['ordPayProvider'];
            $model->ordShipCarrier = $post['ordShipCarrier'];
            $model->ordStatus = 3;
            $model->ordStatusDate = date('Y-m-d h:i:s');
            $model->ordAddInfo = (isset($post['ordAddInfo']) && trim($post['ordAddInfo']) != "") ? $post['ordAddInfo'] : '';
            $model->ordShipType = (isset($post['ordShipType']) && trim($post['ordShipType']) != "") ? $post['ordShipType'] : '';
            $model->ordDate = date('Y-m-d H:i:s');
            
            if(isset($post['ordShipping']) && isset($post['ordShipType']) && trim($post['ordShipping']) != "" && trim($post['ordShipType'])){
                $model->ordShipping = $post['ordShipping'];
                $model->ordShipType = $post['ordShipType'];
                $model->ordTotal = $cart_response['data']['dSubTotal'];
                $settings = Common::getAdminSettings();
                $model->ordHandling = (!empty($settings)) ? $settings->adminHandling : 0;
            }
            

            if ($post['shipToDifferentAddress'] == 1) {
                $model->ordShipExtra1 = (isset($post['ordShipCompanyName']) && trim($post['ordShipCompanyName']) != "") ? $post['ordShipCompanyName'] : '';
                $model->ordShipName = (isset($post['ordShipName']) && trim($post['ordShipName']) != "") ? $post['ordShipName'] : '';
                $model->ordShipAddress = (isset($post['ordShipAddress']) && trim($post['ordShipAddress']) != "") ? $post['ordShipAddress'] : '';
                $model->ordShipCity = (isset($post['ordShipCity']) && trim($post['ordShipCity']) != "") ? $post['ordShipCity'] : '';
                $model->ordShipCountry = (isset($post['ordShipCountry']) && trim($post['ordShipCountry']) != "") ? $post['ordShipCountry'] : '';
                $model->ordShipState = (isset($post['ordShipState']) && trim($post['ordShipState']) != "") ? $post['ordShipState'] : '';
                $model->ordShipZip = (isset($post['ordShipZip']) && trim($post['ordShipZip']) != "") ? $post['ordShipZip'] : '';
                $model->ordShipPhone = (isset($post['ordShipPhone']) && trim($post['ordShipPhone']) != "") ? $post['ordShipPhone'] : '';
                $model->ordShipExtra2 = (isset($post['ordShipBillingZip']) && trim($post['ordShipBillingZip']) != "") ? $post['ordShipBillingZip'] : '';
            }

            //ordComLoc = 3 when checked shipping destination is a commercial location.
            if ($model->save()) {
                $updateCartQ = Cart::whereRaw('(cartOrderID = 0 OR cartOrderID IS NULL)');
                if($tIsGuest != 1){
                    $addModel = new Address();
                    $addModel->insertOrderAddress($post, $user->clID);
                    $updateCartQ->where(['cartClientID' => $user->clID]);
                }else{
                    $updateCartQ->where(['cartSessionID' => $cartSessionID]);
                }
                $updateCart = $updateCartQ->update(['cartOrderID' => $model->ordID,'cartCompleted' => 1]);
                
                if ($updateCart) {
                    DB::commit();
                    //Send push notification
                    $txParameters = json_encode(['ordID' => $model->ordID]);
                    $data = [
                        'iSenderUserId' => NULL,
                        'iReceiverUserId' => $user->clID,
                        'tNotificationType' => Common::notificationType('product_purchase'),
                        'txParameters' => $txParameters,
                        'vTitle' => 'Product purchase',
                        'vText' => 'Your order is placed successfully with order id ' . $model->ordID,
                    ];
                    Common::sendNotification($data);
                    $params = [
                        'date' => Common::ordinal(date('d')) . ' ' . date('F Y H:i'),
                        'ordID' => $model->ordID,
                        'ordName' => $model->ordName,
                        'ordCompany' => $model->ordCountry,
                        'ordAddress' => $model->ordAddress,
                        'ordCity' => $model->ordCity,
                        'ordState' => $model->ordState,
                        'ordZip' => $model->ordZip,
                        'ordCountry' => $model->ordCountry,
                        'ordPhone' => $model->ordPhone,
                        'ordEmail' => $model->ordEmail,
                        'ordShipCarrier' => ($model->ordShipCarrier != 0 && $model->ordShipCarrier != "") ? 'Ship Collect MY Courier Account' : '', //yes,no ('Ship Collect MY Courier Account' if yes)
                        'view_status_url' => LIVE_URL.'orderstatus.php',
                        'cart' => ($cart_response['status']) ? $cart_response['data'] : [] 
                    ];

                    $to = $post['ordEmail'];
                    $from = SMTP_EMAIL;
                    $from_name = SITENAME;
                    $subject = 'Thank you ' . $params['ordName'] . ' for your order ' . $params['ordID'] . ' from ' . SITENAME_SHORT . '.';
                    $body = Common::mailTemplate('orderreceipt', $params);

                    Common::send_mail($to, $from, $from_name, $subject, $body,[CLIENT_EMAIL]);
                    
                    return ['status' => true, 'message' => 'operation_success'];
                } else {
                    return ['status' => false, 'message' => 'something_wrong'];
                }
            } else {
                return ['status' => false, 'message' => 'something_wrong'];
            }
        } catch (Exception $ex) {
            DB::rollback();
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Get order status details
     */

    public function getOrderStatus($post) {
        try {
            $order = DB::table('orders as o')
                    ->select('o.ordStatus', DB::raw('IFNULL(o.ordStatusDate,"") as ordStatusDate'), 'os.statPublic')
                    ->selectRaw("IF(ordTrackNum IS NULL,'',ordTrackNum) as ordTrackNum")
                    ->leftJoin('orderstatus as os', 'os.statID', '=', 'o.ordStatus')
                    ->where('o.ordID', $post['ordID'])
                    ->where('o.ordEmail', $post['ordEmail'])
                    ->first();

            if (!empty($order)) {
                $response = [
                    'status' => true,
                    'message' => 'Track order status.',
                    'data' => $order
                ];
            } else {
                return ['status' => false, 'message' => 'no_record'];
            }
        } catch (Exception $ex) {
            DB::rollback();
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * guest checkout
     */

    public function guestCheckout($post) {
        DB::beginTransaction();
        try {

            $model = new Order();
            $model->ordClientID = 0;
            $model->ordExtra1 = $post['ordCompany'];
            $model->ordCountry = $post['ordCountry'];
            $model->ordName = $post['ordName'];
            $model->ordAddress = $post['ordAddress'];
            $model->ordCity = $post['ordCity'];
            $model->ordState = $post['ordState'];
            $model->ordZip = $post['ordZip'];
            $model->ordPhone = $post['ordPhone'];
            $model->ordEmail = $post['ordEmail'];
            $model->ordTransID = $post['ordTransID'];
            $model->ordPayProvider = $post['ordPayProvider'];
            $model->ordShipCarrier = $post['ordShipCarrier'];
            $model->ordStatus = 3;
            $model->ordStatusDate = date('Y-m-d h:i:s');
            $model->ordAddInfo = (isset($post['ordAddInfo']) && trim($post['ordAddInfo']) != "") ? $post['ordAddInfo'] : '';
            $model->ordShipType = (isset($post['ordShipType']) && trim($post['ordShipType']) != "") ? $post['ordShipType'] : '';

            if ($post['shipToDifferentAddress'] == 1) {
                $model->ordShipExtra1 = (isset($post['ordShipCompanyName']) && trim($post['ordShipCompanyName']) != "") ? $post['ordShipCompanyName'] : '';
                $model->ordShipName = (isset($post['ordShipName']) && trim($post['ordShipName']) != "") ? $post['ordShipName'] : '';
                $model->ordShipAddress = (isset($post['ordShipAddress']) && trim($post['ordShipAddress']) != "") ? $post['ordShipAddress'] : '';
                $model->ordShipCity = (isset($post['ordShipCity']) && trim($post['ordShipCity']) != "") ? $post['ordShipCity'] : '';
                $model->ordShipCountry = (isset($post['ordShipCountry']) && trim($post['ordShipCountry']) != "") ? $post['ordShipCountry'] : '';
                $model->ordShipState = (isset($post['ordShipState']) && trim($post['ordShipState']) != "") ? $post['ordShipState'] : '';
                $model->ordShipZip = (isset($post['ordShipZip']) && trim($post['ordShipZip']) != "") ? $post['ordShipZip'] : '';
                $model->ordShipPhone = (isset($post['ordShipPhone']) && trim($post['ordShipPhone']) != "") ? $post['ordShipPhone'] : '';
                $model->ordShipExtra2 = (isset($post['ordShipBillingZip']) && trim($post['ordShipBillingZip']) != "") ? $post['ordShipBillingZip'] : '';
            }
            if ($model->save()) {

                /*                 * ************************************** */
//                $cartSessionID = md5(microtime(true));
                $cart_json = json_decode($post['cart_json'], true);

                if (json_last_error() != 0) {
                    return ['status' => false, 'message' => 'invalid json string.'];
                }

                if (!empty($cart_json)) {
                    foreach ($cart_json as $key => $val) {
                        $product = Product::where(['pID' => $val['cartProdID']])->first();
                        if (!empty($product)) {
                            $option = [];
                            //getting product option data
                            if (trim($val['coOptID']) != "" && trim($val['coOptID']) != 0) {
                                $option = Option::from('options AS o')
                                        ->selectRaw('o.optID,o.optName,o.optPriceDiff,o.optWeightDiff,og.optGrpName')
                                        ->leftjoin('optiongroup AS og', 'og.optGrpID', '=', 'o.optGroup')
                                        ->where(['optID' => $val['coOptID']])
                                        ->first();
                                if (empty($option)) {
                                    return ['status' => false, 'message' => 'invalid_option'];
                                }
                            }
                            //Adding product to cart
                            $modelCart = new Cart();
                            $modelCart->cartProdID = $val['cartProdID'];
                            $modelCart->cartProdName = $product->pName;
                            $modelCart->cartProdPrice = $product->pPrice;
                            $modelCart->cartDateAdded = date('Y-m-d H:i:s');
                            $modelCart->cartQuantity = $val['cartQuantity'];
                            $modelCart->cartClientID = 0;
                            $modelCart->cartOrderID = $model->ordID;
                            $modelCart->cartCompleted = 1;
                            if ($modelCart->save()) {
                                if (!empty($option) && trim($val['coOptID']) != "" && trim($val['coOptID']) != 0) {
                                    $cartoptionModel = new Cartoption();
                                    $cartoptionModel->coCartID = $modelCart->cartID;
                                    $cartoptionModel->coOptID = $val['coOptID'];
                                    $cartoptionModel->coOptGroup = $option->optGrpName;
                                    $cartoptionModel->coCartOption = $option->optName;
                                    $cartoptionModel->coPriceDiff = $option->optPriceDiff;//round(($option->optPriceDiff * $product->pPrice) / 100, 2);
                                    $cartoptionModel->coWeightDiff = (($product->pWeight * $option->optWeightDiff) / 100);
                                    $cartoptionModel->save();
                                }
                            } else {
                                return ['status' => false, 'message' => 'something_wrong'];
                            }
                        } else {
                            return ['status' => false, 'message' => 'invalid_product'];
                        }
                    }
                    $result = [
                        'ordID' => $model->ordID,
                        'ordEmail' => $model->ordEmail
                    ];
                    
                    $cartObj = new Cart();
                    $cart_response = $cartObj->cart_list($post,$model->ordID,true);
                    
                    $params = [
                        'date' => Common::ordinal(date('d')) . ' ' . date('F Y H:i'),
                        'ordID' => $model->ordID,
                        'ordName' => $model->ordName,
                        'ordCompany' => $model->ordCountry,
                        'ordAddress' => $model->ordAddress,
                        'ordCity' => $model->ordCity,
                        'ordState' => $model->ordState,
                        'ordZip' => $model->ordZip,
                        'ordCountry' => $model->ordCountry,
                        'ordPhone' => $model->ordPhone,
                        'ordEmail' => $model->ordEmail,
                        'ordShipCarrier' => ($model->ordShipCarrier != 0 && $model->ordShipCarrier != "") ? 'Ship Collect MY Courier Account' : '', //yes,no ('Ship Collect MY Courier Account' if yes)
                        'view_status_url' => LIVE_URL.'orderstatus.php',
                        'cart' => ($cart_response['status']) ? $cart_response['data'] : []
                    ];
                    
                    $to = $model->ordEmail;
                    $from = SMTP_EMAIL;
                    $from_name = SITENAME;
                    $subject = 'Thank you ' . $model->ordName . ' for your order ' . $model->ordID . ' from ' . SITENAME_SHORT . '.';
                    $body = Common::mailTemplate('orderreceipt', $params);
                    
                    $mailRes = Common::send_mail($to, $from, $from_name, $subject, $body);
                    
                    DB::commit();
                    return ['status' => true, 'message' => 'operation_success', 'data' => $result];
                }
            } else {
                return ['status' => false, 'message' => 'something_wrong'];
            }
        } catch (Exception $ex) {
            DB::rollback();
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

}
