<?php

use Illuminate\Database\Eloquent\Model as Eloquent;
use Illuminate\Database\Capsule\Manager as DB;

class Product extends Eloquent {

    protected $table = 'products';
    protected $primaryKey = null;
    public $timestamps = false;

    public function product_images() {
        return $this->hasMany(Productimage::class, 'imageProduct', 'pID')->where('imageType', 0);
    }

    public function product_list($post) {
        $response = [];
        try {
            $list = Product::from('products AS p')
                    ->selectRaw('p.pID,p.pName,p.pPrice,p.pDescription2 as pDescription,IF(p.pDisplay = 0 OR p.pSell = 0,0,1) AS pDisplay,
                            IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = p.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = p.pID ORDER BY pi.imageNumber LIMIT 1))) AS image
                            
')
                    ->leftjoin('multisections AS m', 'm.pID', '=', 'p.pID')
                    ->where('p.pDisplay', '!=', 0);

            //Category filter
            if (isset($post['sectionID']) && trim($post['sectionID']) != "") {
                $list->whereRaw("(p.pSection = '" . $post['sectionID'] . "' OR m.pSection = '" . $post['sectionID'] . "')");
            }
            //Max price
            if (isset($post['maxPrice']) && trim($post['maxPrice']) != "") {
                $list->where("p.pPrice", "<=", $post['maxPrice']);
            }
            if (isset($post['searchType']) && $post['searchType'] != "") {
                //Text search
                if ($post['searchType'] == 1) {
                    if (isset($post['search_text']) && trim($post['search_text']) != "") {
                        $list->whereRaw("(p.pID LIKE '%" . $post['search_text'] . "%' OR p.pName LIKE '%" . $post['search_text'] . "%')");
                    }
                } else if ($post['searchType'] == 2) {
                    if (isset($post['search_text']) && trim($post['search_text']) != "") {
                        $words = str_replace(" ", ",", $post['search_text']);
                        $list->whereRaw("MATCH(p.pName) AGAINST('" . $words . "' IN BOOLEAN MODE)");
                    }
                } else {
                    if (isset($post['search_text']) && trim($post['search_text']) != "") {
                        $list->whereRaw("BINARY p.pName = '" . $post['search_text'] . "'");
                    }
                }
            }
            $list->groupBy("p.pID");
            $list->orderBy("p.pOrder");
//            $list->orderBy("p.pName")->orderBy("p.pID");
            //Pagination
            $totalRecord = $list->get()->count();
            /* $reqPageno = (isset($post['pageno']) && trim($post['pageno']) != "") ? $post['pageno'] : 1;
              $recordPerPage = (isset($post['count'])) ? $post['count'] : RECORD_PER_PAGE;
              $recPages = ($totalRecord == 0) ? 0 : ($totalRecord < $recordPerPage) ? 1 : $totalRecord / $recordPerPage;
              $pageno = ($reqPageno - 1) * $recordPerPage;
              $nextPage = ($reqPageno < ceil($recPages)) ? $reqPageno + 1 : 0;

              $list->offset($pageno)->limit($recordPerPage); */
            $result = $list->get();

            $response = [
                'status' => true,
                'message' => 'Product list',
                'totalRecord' => $totalRecord,
                'totalPage' => ceil($recPages),
                'nextPage' => $nextPage,
                'data' => $result
            ];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }
    
    private function get_recursive_sections($section_id){
        $arr = [];

        $sections_arr = Section::selectRaw("sectionID,rootSection")->whereRaw("sectionDisabled <= 0 AND (topSection IN('".$section_id."') OR (sectionID IN ('".$section_id."') AND rootSection=1))")->get();
        foreach ($sections_arr as $key => $val){
            if($val->rootSection != 0){
                $arr[] = $val->sectionID;
            }else{
                $ids = $this->get_recursive_sections($val->sectionID);
                if(!empty($ids)){
                    $arr = array_merge($arr,$ids);
                }
            }
        }
        return $arr;
    }

    public function search_product($post) {
        $response = [];
        try {
            $sql_price = $sql_category = "";
            $product_join = " (products INNER JOIN sections on products.pSection=sections.sectionID) ";
            
            //Category filter
            if (isset($post['sectionID']) && trim($post['sectionID']) != "") {
                $product_join = " (multisections RIGHT JOIN products ON products.pID=multisections.pID INNER JOIN sections on products.pSection=sections.sectionID) ";
                $all_section = $this->get_recursive_sections($post['sectionID']);
                $sectionids_list = implode(",", $all_section);
                
                $sql_category = " AND (products.pSection IN (".$sectionids_list.") OR multisections.pSection IN (".$sectionids_list.")) ";
            }
            
            //Max price
            if (isset($post['maxPrice']) && trim($post['maxPrice']) != "") {
                $sql_price = " AND pPrice <= '".$post['maxPrice']."' ";
            }
            $search_text_flag = true;
            if (isset($post['searchType']) && $post['searchType'] != "") {
                $words = explode(",", str_replace(" ", ",", $post['search_text']));
                //Text search
                if ($post['searchType'] == 1) {
                    $sql = "";
                    $u_sql = "";
                    if (isset($post['search_text']) && trim($post['search_text']) != "") {
                        if (!empty($words)) {
                            $sql = $u_sql = " AND (";
                            foreach ($words as $key => $val) {
                                $sql .= "(products.pID LIKE '%" . $val . "%' OR pName LIKE '%" . $val . "%' OR pSKU LIKE '%" . $val . "%' OR pSearchParams LIKE '%" . $val . "%')";
                                $sql .= " AND ";
                                $u_sql .= " (pDescription2 LIKE '%" . $val . "%' OR pLongDescription2 LIKE '%" . $val . "%')";
                                $u_sql .= " AND ";
                            }
                            $sql = substr_replace($sql, "", -4);
                            $u_sql = substr_replace($u_sql, "", -4);
                            $sql .= ")";
                            $u_sql .= ")";
                        }
                        $final_query = 'SELECT SQL_CALC_FOUND_ROWS DISTINCT 0 AS relevanceorder,products.pID,pName,pPrice,pOrder,products.pDescription2 as pDescription,IF(products.pDisplay = 0 OR products.pSell = 0,0,1) AS pDisplay,
IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1))) AS image
    
FROM '.$product_join.' 

WHERE sectionDisabled<=0 AND pDisplay<>0 ' . $sql_category . $sql_price . $sql . ' UNION ALL SELECT DISTINCT 1 AS relevanceorder,products.pID,pName,pPrice,pOrder,products.pDescription2 as pDescription,IF(products.pDisplay = 0 OR products.pSell = 0,0,1) AS pDisplay,
    IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1))) AS image
        
                    FROM '.$product_join.' 
                    
WHERE sectionDisabled<=0 AND pDisplay<>0 '. $sql_category . $sql_price . $u_sql . ' AND NOT products.pID IN (SELECT products.pID FROM (products INNER JOIN sections on products.pSection=sections.sectionID) WHERE sectionDisabled<=0 AND pDisplay<>0 ' . $sql . ') ORDER BY 1,pOrder,pID';
                        $search_text_flag = false;
                    }
                } else if ($post['searchType'] == 2) {
                    $sql = "";
                    if (isset($post['search_text']) && trim($post['search_text']) != "") {
                        if (isset($post['search_text']) && trim($post['search_text']) != "") {

                            if (!empty($words)) {
                                $sql = " AND (";
                                foreach ($words as $key => $val) {
                                    $sql .= "(products.pID LIKE '%" . $val . "%' OR pName LIKE '%" . $val . "%' OR pSKU LIKE '%" . $val . "%' OR pSearchParams LIKE '%" . $val . "%')";
                                    $sql .= " OR ";
                                }
                                $sql = substr_replace($sql, "", -4);
                                $sql .= ")";
                            }
                            
                            $final_query = 'SELECT SQL_CALC_FOUND_ROWS DISTINCT 0 AS relevanceorder,products.pID,pName,pPrice,pOrder,products.pDescription2 as pDescription,IF(products.pDisplay = 0 OR products.pSell = 0,0,1) AS pDisplay,
IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1))) AS image 
                                    FROM '.$product_join.'
                                    WHERE 
                                    sectionDisabled<=0 AND pDisplay<>0 
                                    '. $sql_category . $sql_price . $sql.'
                                    ORDER BY 1,pOrder,pID';
                            
                        }
                        $search_text_flag = false;
                    }
                } else {
                    $sql = "";
                    $u_sql = "";
                    if (isset($post['search_text']) && trim($post['search_text']) != "") {
                        
                            $sql = $u_sql = " AND (";
                            
                            $sql .= "products.pID LIKE '%" . $post['search_text'] . "%' OR pName LIKE '%" . $post['search_text'] . "%' OR pSKU LIKE '%" . $post['search_text'] . "%' OR pSearchParams LIKE '%" . $post['search_text'] . "%'";
                            $sql .= " OR ";
                            $u_sql .= " pDescription2 LIKE '%" . $post['search_text'] . "%' OR pLongDescription2 LIKE '%" . $post['search_text'] . "%'";
                            $u_sql .= " OR ";
                            
                            $sql = substr_replace($sql, "", -4);
                            $u_sql = substr_replace($u_sql, "", -4);
                            $sql .= ")";
                            $u_sql .= ")";
                        
                        $final_query = 'SELECT SQL_CALC_FOUND_ROWS DISTINCT 0 AS relevanceorder,products.pID,pName,pPrice,pOrder,products.pDescription2 as pDescription,IF(products.pDisplay = 0 OR products.pSell = 0,0,1) AS pDisplay,
IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1))) AS image
    
FROM '.$product_join.' 

WHERE sectionDisabled<=0 AND pDisplay<>0 ' . $sql_category . $sql_price . $sql . ' UNION ALL SELECT DISTINCT 1 AS relevanceorder,products.pID,pName,pPrice,pOrder,products.pDescription2 as pDescription,IF(products.pDisplay = 0 OR products.pSell = 0,0,1) AS pDisplay,
    IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1))) AS image
        
                    FROM '.$product_join.'
                    
                    WHERE sectionDisabled<=0 AND pDisplay<>0 ' . $u_sql . ' AND NOT products.pID IN (SELECT products.pID FROM (products INNER JOIN sections on products.pSection=sections.sectionID) WHERE sectionDisabled<=0 AND pDisplay<>0 '. $sql_category . $sql_price . $sql . ') ORDER BY 1,pOrder,pID';
                        
                        $search_text_flag = false;
                    }
                }
            }
            if($search_text_flag){
                $final_query = 'SELECT SQL_CALC_FOUND_ROWS DISTINCT 0 AS relevanceorder,products.pID,pName,pPrice,pOrder,products.pDescription2 as pDescription,IF(products.pDisplay = 0 OR products.pSell = 0,0,1) AS pDisplay,
IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = products.pID ORDER BY pi.imageNumber LIMIT 1))) AS image FROM '.$product_join.' WHERE sectionDisabled<=0 AND pDisplay<>0 '. $sql_category . $sql_price .' ORDER BY pOrder,pID';
            }
            $result = DB::select($final_query);

            $response = [
                'status' => true,
                'message' => 'Search product list',
                'totalRecord' => count($result),
                'totalPage' => 0,
                'nextPage' => 0,
                'data' => $result
            ];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Product detail
     */

    public function product_detail($post) {
        $response = [];
        try {
            $product_id = $post['pID'];
            $product_detail = Product::from('products AS p')
                    ->selectRaw('p.pID,p.pName,p.pPrice,IF(p.pDisplay = 0 OR p.pSell = 0,0,1) AS pDisplay')
                    ->selectRaw("REPLACE(p.pDescription2,'\"','\'') AS pDescription")
                    ->selectRaw("REPLACE(p.pLongDescription2,'\"','\'') AS pLongdescription")
                    ->with(['product_images' => function($query) {
                            $query->selectRaw("imageProduct,IF(imageSrc IS NULL,'', IF(imageSrc LIKE '%//%',REPLACE(imageSrc,'//','https://'),CONCAT('" . LIVE_URL . "','',imageSrc))) AS imageSrc");
                        }])
                    ->where(['p.pID' => $product_id])
                    ->first();

            if (!empty($product_detail)) {
                $productOption = Prodoption::from('prodoptions AS po')
                        ->selectRaw('og.optGrpID,og.optGrpName,og.optFlags')
                        ->leftjoin('optiongroup AS og', 'og.optGrpID', '=', 'po.poOptionGroup')
                        ->where(['po.poProdID' => $product_id])
                        ->first();

                if (!empty($productOption)) {
                    //product options list
                    $options = Option::from('options as o')
                            ->selectRaw("o.optID,o.optName,o.optPriceDiff,o.optWeightDiff,o.optRegExp,
                            IF(
                                TRIM(optRegExp) != '',
                                (SELECT pPrice FROM products as p WHERE p.pID = o.optRegExp),
                                IF(
                                    " . $productOption->optFlags . " = 0,
                                    (" . $product_detail->pPrice . " + o.optPriceDiff),
                                    " . $product_detail->pPrice . " + ((" . $product_detail->pPrice . " * o.optPriceDiff)/100)
                                )
                            ) AS optPrice")
                            ->where(['o.optGroup' => $productOption->optGrpID])
                            ->get();

                    $productOption->options = ($options->count() > 0) ? $options : [];
                }
                $product_detail->productOptions = (!empty($productOption)) ? $productOption : (object) [];
                $product_detail->productImages = (!empty($product_detail->product_images)) ? $product_detail->product_images : [];
                unset($product_detail->product_images);
            }
            $response = [
                'status' => true,
                'message' => 'Product detail',
                'data' => $product_detail
            ];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Custom quotation request
     */

    public function customQuotationRequest($post) {
        $response = [];
        try {

            $to = SMTP_EMAIL;
            $from = $post['vEmail'];
            $from_name = $post['vFirstName'] . ' ' . $post['vLastName'];
            $subject = SITENAME;
            $body = Common::mailTemplate('custom_quote', $post);

            $mailRes = Common::send_mail($to, $from, $from_name, $subject, $body);
            if ($mailRes) {
                return ['status' => true, 'message' => 'custom_quote_sent'];
            } else {
                return ['status' => true, 'message' => 'something_wrong'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Vci selection help
     */

    public function vciSelectionHelp($post) {
        $response = [];
        try {
            $to = SMTP_EMAIL;
            $from = $post['vEmail'];
            $from_name = $post['vName'];
            $subject = 'VCI Selection Help Request ' . $post['vSubject'];
            $body = Common::mailTemplate('vci_selection_help', $post);

            $mailRes = Common::send_mail($to, $from, $from_name, $subject, $body);
            if ($mailRes) {

                $subject_user = 'VCI selection help form confirmaiton';
                $body_user = Common::mailTemplate('vci_selection_confirmation', $post);
                $mailRes = Common::send_mail($post['vEmail'], SMTP_EMAIL, SITENAME, $subject_user, $body_user);

                return ['status' => true, 'message' => 'vci_selection_help_sent'];
            } else {
                return ['status' => true, 'message' => 'something_wrong'];
            }
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

}
