<?php

use Illuminate\Database\Eloquent\Model as Eloquent;
use Illuminate\Database\Capsule\Manager as DB;

class Section extends Eloquent {

    protected $table = 'sections';
    protected $primaryKey = 'sectionID';
    public $timestamps = false;

    public function category_list() {
        $response = [];
        try {
            $result = Section::from('sections AS s')
                    ->select(["s.sectionID", "s.sectionName"])
                    ->selectRaw("
                                        (
                                            CASE WHEN EXISTS(SELECT NULL FROM sections AS sc WHERE sc.topSection = s.sectionID AND sc.sectionDisabled <= 0)
                                            THEN 1
                                            ELSE 0
                                            END
                                        ) AS isSubCategory
                                    ")
                    ->where(['s.topSection' => 0])
                    ->orderBy("s.sectionOrder")
                    ->get();

            $response = ['status' => true, 'message' => 'Category list.', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Sub category list
     */

    public function sub_category_list($post) {
        $response = [];
        try {
            $result = Section::from('sections AS s')
                    ->selectRaw("s.sectionID, s.sectionName,IF(s.sectionImage IS NULL,'',CONCAT('" . LIVE_URL . "','',s.sectionImage)) AS sectionImage")
                    ->selectRaw("
                                        (
                                            CASE WHEN EXISTS(SELECT NULL FROM sections AS sc WHERE sc.topSection = s.sectionID AND sc.sectionDisabled <= 0)
                                            THEN 1
                                            ELSE 0
                                            END
                                        ) AS isSubCategory
                                    ")
                    ->where(['s.topSection' => $post['sectionID']])
                    ->where('s.sectionDisabled', '<=', 0)
                    ->orderBy("s.sectionOrder")
                    ->get();

            $response = ['status' => true, 'message' => 'Sub category list.', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Go shopping list
     */

    public function go_shopping() {
        $response = [];
        try {
            //main category
            $sections = Section::from('sections AS s')
                    ->select(["s.sectionID", "s.sectionName"])
                    ->selectRaw("(
                                            CASE WHEN EXISTS(SELECT NULL FROM sections AS sc WHERE sc.topSection = s.sectionID AND sc.sectionDisabled <= 0)
                                            THEN 1
                                            ELSE 0
                                            END
                                        ) AS isSubCategory
                                    ")
                    ->where(['s.topSection' => 0])
                    ->orderBy("s.sectionOrder")
                    ->get();

            $result = [];
            if (!empty($sections)) {
                foreach ($sections as $key => $val) {

                    if ($val->isSubCategory == 0) {
                        //Product list
                        $product_list = Product::from('products AS p')
                                ->selectRaw('p.pID AS id,p.pName AS name,
                                        
                                        IF((SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = p.pID ORDER BY pi.imageNumber LIMIT 1) IS NULL,"",CONCAT("' . LIVE_URL . '","",(SELECT pi.imageSrc FROM productimages AS pi WHERE pi.imageProduct = p.pID ORDER BY pi.imageNumber LIMIT 1))) AS image,0 as isSubCategory
                                        ')
                                ->leftjoin('multisections AS m', 'm.pID', '=', 'p.pID')
                                ->where('p.pDisplay', '!=', 0)
                                ->whereRaw("(p.pSection = " . $val->sectionID . " OR m.pSection = " . $val->sectionID . ")")
                                ->groupBy("p.pID")
                                ->orderBy("p.pOrder")
                                ->limit(3)
                                ->get();
                        if (!empty($product_list) && count($product_list) > 0) {
                            $val->list = $product_list;
                            $result[] = $val;
                        }
                    } else {
                        //Sub category list
                        $val->list = Section::from('sections AS s')
                                ->selectRaw('s.sectionID AS id, s.sectionName as name,CONCAT("' . LIVE_URL . '","",sectionImage) AS image')
                                ->selectRaw('(
                                            CASE WHEN EXISTS(SELECT NULL FROM sections AS sc WHERE sc.topSection = s.sectionID AND sc.sectionDisabled <= 0)
                                            THEN 1
                                            ELSE 0
                                            END
                                        ) AS isSubCategory')
                                ->where(['s.topSection' => $val->sectionID])
                                ->where('s.sectionDisabled', '<=', 0)
                                ->orderBy("s.sectionOrder")
                                ->limit(3)
                                ->get();

                        $result[] = $val;
                    }
                }
            }
            
            $advert = [
                [
                    "url" => LIVE_URL."proddetail.php?prod=nr3100A",
                    "image" => LIVE_URL."prodimages/Nox-Rust-Aerosols.jpg"
                ],
                [
                    "url" => LIVE_URL."vci-paper.php",
                    "image" => LIVE_URL."prodimages/non-ferrous.jpg"
                ],
                [
                    "url" => LIVE_URL."vci-poly-film-bags.php",
                    "image" => LIVE_URL."prodimages/corrosion-protection-vci-film.jpg"
                ],
                [
                    "url" => LIVE_URL."vci-powder.php",
                    "image" => LIVE_URL."prodimages/vci-powder1.jpg"
                ]
            ];
            
            $result = [
                'go_shopping_list' => $result,
                'advert' => $advert
            ];
            
            $response = [
                'status' => true,
                'message' => 'Go shopping list',
                'data' => $result
            ];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Nested category list
     */

    public function nested_category_list() {
        $response = [];
        try {
            $list = Section::selectRaw('sectionID,sectionName,topSection,rootSection')
                    ->where('sectionDisabled', '<=', 0)
                    ->whereRaw('sectionID NOT IN(76,89)')
                    ->orderBy("sectionOrder")
                    ->get();
            //Generating recursive category array
            $result = $this->generateNestedArray($list);
            $response = ['status' => true, 'message' => 'Nasted category list', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
        return $response;
    }

    /*
     * Generate multi level menu
     */

    public function generateNestedArray($elements, $parentId = 0, $num = 0) {
        $branch = array();
        foreach ($elements as $key => $element) {
            if ($element->topSection == $parentId) {
                $children = $this->generateNestedArray($elements, $element->sectionID,$num + 1);
                
                if ($children) {
                    $element['num'] = $num;
                    $element['sub_category'] = $children;
                    
                } else {
                    $element['num'] = $num;
                    $element['sub_category'] = [];
                }
                $branch[] = $element;
            }
        }
        
        return $branch;
    }

    //Product technical sheet
    public function productTechnicalSheet() {
        try {
            $pathdir = '../../../pdf';
            $path = PDF_FILE_URL;
            $files = scandir($pathdir);

            $descriptions = [];
            $descriptionsFile = file($pathdir . '/' . 'descript.ion');
            for ($i = 0; $i < count($descriptionsFile); $i++) {
                $d = explode("\t", $descriptionsFile[$i]);
                $descriptions[$d[0]] = htmlentities(join("\t", array_slice($d, 1)));
            }
            
            $result = [];
            foreach ($files as $key => $val) {
                $ext = pathinfo($val, PATHINFO_EXTENSION);
                if ($ext == 'pdf') {
                    $desc = (trim($descriptions[strtolower($val)]) != "") ? $descriptions[strtolower($val)] : $descriptions[$val];
                    $result[] = [
                        'name' => $val,
                        'date' => date("d-m-Y", filemtime($pathdir . '/' . $val)),
                        'description' => $desc,
                        'size' => filesize($pathdir . '/' . $val),
                        'type' => $ext,
                        'link' => $path . $val
                    ];
                }
            }
            return ['status' => true, 'message' => 'Product technical sheet', 'data' => $result];
        } catch (Exception $ex) {
            return ['status' => false, 'message' => (THROW_EXCEPTION) ? $ex->getMessage() : 'something_wrong'];
        }
    }

}
