<?php

/**
 * @SWG\Get(path="/cart_list",
 *     summary="Shopping cart list",
 *     @SWG\Parameter(
 *         name="clID",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default="586",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "Cart list response",
 *         examples={
 *                   "application/json": {
 *                       "message": "Cart list",
 *                       "data": {
 *                           "dSubTotal": 1142.95,
 *                           "dShippingHandlingCharge": 57.99,
 *                           "dGrandTotal": 1200.94,
 *                           "cartList": {
 *                               {
 *                                   "cartID": 22873,
 *                                   "cartProdID": "KPR-rust-remover-55gal",
 *                                   "cartProdName": "KPR Rust Remover 55 Gal",
 *                                   "cartProdPrice": 791,
 *                                   "cartQuantity": 1,
 *                                   "cartOrderID": 0,
 *                                   "pExemptions": 0,
 *                                   "pWeight": 0,
 *                                   "imageSrc": "//www.static.corrosionvci.com/prodimages/rust-remover-drum.jpg",
 *                                   "dTotalPrice": 791
 *                               },
 *                               {
 *                                   "cartID": 22874,
 *                                   "cartProdID": "vci9x12zip",
 *                                   "cartProdName": "VCI 9x12 Zipper Bags",
 *                                   "cartProdPrice": 210,
 *                                   "cartQuantity": 1,
 *                                   "cartOrderID": 0,
 *                                   "pExemptions": 0,
 *                                   "pWeight": 47,
 *                                   "imageSrc": "//www.static.corrosionvci.com/prodimages/zipper-bags.jpg",
 *                                   "dTotalPrice": 210
 *                               }
 *                           }
 *                       }
 *                   }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         @SWG\Schema(
 *                ref="#/definitions/ErrorResponse"
 *              ), 
 *         examples={
 *                   "application/json": {
 *                         "message": "Cart list.",
 *                         "data": {}
 *                   }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = []; //$_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('GET');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post, false);
    if ($response['status'] == 200) {
        //User Authentication
        $response = Authentication::auth_user();
        if ($response['status'] == 200) {
            $model = new Cart();
            $response = $model->cart_list();
        }
    }
}
_response($response);
