<?php

/**
 * @SWG\Post(path="/product_detail",
 *     summary="Product detail",
 *     consumes={"application/x-www-form-urlencoded"},
 *     @SWG\Parameter(
 *         description="Product id",
 *         in="formData",
 *         name="pID",
 *         required=true,
 *         type="string",
 *         format="string",
 *         default="VCI-4x1000",
 *     ),
 *     @SWG\Parameter(
 *         name="nonce",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=NONCE,
 *     ),
 *      @SWG\Parameter(
 *         name="timestamp",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TIMESTAMP,
 *     ),
 *      @SWG\Parameter(
 *         name="token",
 *         in="header",
 *         required=true,
 *         type="string",
 *         default=TOKEN,
 *     ),
 *     @SWG\Response(
 *         response = 200,
 *         description = "User collection response",
 *         examples={
 *                   "application/json": {
 *                           "message": "Product detail",
 *                           "data": {
 *                               "pID": "Mil-PRF-16173",
 *                               "pName": "Mil-PRF-16173E Grade 4 Class 1",
 *                               "pPrice": 225.21,
 *                               "pDescription": "Desc",
 *                               "pLongdescription": "Desc",
 *                               "product_options": {
 *                                   "optGrpID": 11,
 *                                   "optGrpName": "Nox-Rust Sizes",
 *                                   "options": {
 *                                       {
 *                                           "optName": "5 Gallon Pail",
 *                                           "optPriceDiff": 0,
 *                                           "prc": 225.21
 *                                       },
 *                                       {
 *                                           "optName": "Six Aerosol Cans -Ground ONLY",
 *                                           "optPriceDiff": -46.66,
 *                                           "prc": 178.55
 *                                       }
 *                                   }
 *                               },
 *                               "product_images": {
 *                                   {
 *                                       "imageProduct": "Mil-PRF-16173",
 *                                       "imageSrc": "//www.static.corrosionvci.com/prodimages/mil-prf-16173.jpg"
 *                                   }
 *                               }
 *                           }
 *                       }
 *          },
 *     ),
 *    @SWG\Response(
 *         response = 400,
 *         description = "Confilct",
 *         examples={
 *                   "application/json": {
 *                   "message": "Product list",
 *                   "totalRecord": 0,
 *                   "totalPage": 0,
 *                   "nextPage": "",
 *                   "data": {}
 *               }
 *         },
 *     ),
 *     @SWG\Response(
 *         response="404",
 *         description="page not found"
 *     ),
 * )
 */
$response = [];
$post = $_POST;

$obj = new Common();
//Content type validation
$response = $obj->checkContentType('POST');
if ($response['status'] == 200) {
    //Nonce-Token & request validation
    $response = $obj->apiRequestCheck($post);
    if ($response['status'] == 200) {
        //Validation rules
        $requiredFields = [
            'pID' => 'required',
        ];
        $response = $obj->fieldValidation($post, $requiredFields);
        if ($response['status'] == 200) {
            $model = new Product();
            $response = $model->product_detail($post);
        }
    }
}
_response($response);
