<?php

use ActiveRecord\ConnectionManager;
use ActiveRecord\Model;

class Customerlogin extends Model {

    static $table_name = 'customerlogin';

    public function login($post) {
        $response = [];
        try {
            $user = Customerlogin::find('first', [
                        'select' => "*",
                        'conditions' => [
                            'clEmail' => $post['clEmail'],
                            'clPW' => md5(MD5_HASH_KEY . $post['clPW'])
                        ]
            ]);
            //validation use exist or not
            if (!empty($user)) {
                $device = Device::find('first', [
                            'select' => "*",
                            'conditions' => [
                                'clID' => $user->clid
                            ]
                ]);
                //Updating device information
                if (!empty($device)) {
                    $device->txdevicetoken = $post['txDeviceToken'];
                    $device->tdeviceos = $post['tDeviceOs'];
                    $device->vdevicename = (isset($post['vDeviceName']) && trim($post['vDeviceName']) != "") ? $post['vDeviceName'] : '';
                    $device->vresolution = (isset($post['vResolution']) && trim($post['vResolution']) != "") ? $post['vResolution'] : '';
                    $device->vosversion = (isset($post['vOsVersion']) && trim($post['vOsVersion']) != "") ? $post['vOsVersion'] : '';
                    $device->created_at = date('Y-m-d H:i:s');
                    $device->save();
                }

                $result = $this->userResponse($user);
                $response = ['status' => true, 'message' => 'login_success', 'data' => $result];
            } else {
                $response = ['status' => false, 'message' => 'wrong_email_password'];
            }
        } catch (Exception $ex) {
            $response = ['status' => false, 'message' => $ex->getMessage()];
        }
        return $response;
    }

    /*
     * Signup Customer
     */

    public function signup($post) {
        $response = [];
        $trans = ConnectionManager::get_connection();
        $trans->transaction();

        try {
            //Check email already exist or not
            if ($this->isEmailRegistered($post['clEmail'])) {
                //Create customer
                $model = new Customerlogin();
                $model->clusername = $post['clUserName'];
                $model->clpw = md5(MD5_HASH_KEY . $post['clPW']);
                $model->clemail = $post['clEmail'];
                $model->cldatecreated = date('Y-m-d H:i:s');
                $model->save();

                //Add device entry
                $deviceModel = new Device();
                $deviceModel->clid = $model->clid;
                $deviceModel->txdevicetoken = $post['txDeviceToken'];
                $deviceModel->tdeviceos = $post['tDeviceOs'];
                $deviceModel->vdevicename = (isset($post['vDeviceName']) && trim($post['vDeviceName']) != "") ? $post['vDeviceName'] : '';
                $deviceModel->vresolution = (isset($post['vResolution']) && trim($post['vResolution']) != "") ? $post['vResolution'] : '';
                $deviceModel->vosversion = (isset($post['vOsVersion']) && trim($post['vOsVersion']) != "") ? $post['vOsVersion'] : '';
                $deviceModel->updated_at = date('Y-m-d H:i:s');
                $deviceModel->created_at = date('Y-m-d H:i:s');
                $deviceModel->save();

                $result = $this->userResponse($model);

                $trans->commit();
                $response = ['status' => true, 'message' => 'signup_success', 'data' => $result];
            } else {
                $response = ['status' => false, 'message' => 'email_already_exist'];
            }
        } catch (Exception $ex) {
            $trans->rollback();
            $response = ['status' => false, 'message' => $ex->getMessage()];
        }
        return $response;
    }

    /*
     * Email already exist check
     */

    private function isEmailRegistered($clEmail) {
        $user = Customerlogin::find('first', [
                    'select' => "count(*) AS count",
                    'conditions' => [
                        'clEmail' => $clEmail
                    ]
        ]);
        if (empty($user) || $user->count == 0) {
            return true;
        } else {
            return false;
        }
    }

    /*
     * Build user common response
     */

    private function userResponse($model) {
        if (!empty($model)) {
            return [
                'clID' => $model->clid,
                'clUserName' => $model->clusername,
                'clEmail' => $model->clemail,
            ];
        } else {
            return (object) [];
        }
    }

    /*
     * Forgot password mail send
     */

    public function forgotpassword($post) {
        $response = [];
        try {
            $user = Customerlogin::find('first', [
                        'select' => "*",
                        'conditions' => [
                            'clEmail' => trim($post['clEmail'])
                        ]
            ]);
            //Validate email exist or not
            if (!empty($user)) {
                $params = [
                    'clEmail' => $user->clemail,
                    'clPW' => $user->clpw
                ];

                $to = $post['clEmail'];
                $from = SMTP_EMAIL;
                $from_name = SITENAME;
                $subject = APP_NAME . ' | Forgot password';
                $body = Common::mailTemplate('forgotpassword', $params);

                $mailRes = Common::send_mail($to, $from, $from_name, $subject, $body);
                if ($mailRes) {
                    $response = ['status' => true, 'message' => 'Pasword reset mail sent to your email address, Please check your mail inbox.'];
                } else {
                    $response = ['status' => false, 'message' => 'mail_sending_fail'];
                }
            } else {
                $response = ['status' => false, 'message' => 'invalid_email'];
            }
        } catch (Exception $ex) {
            $response = ['status' => false, 'message' => $ex->getMessage()];
        }
        return $response;
    }

    /*
     * Change user password
     */

    public function change_password($post) {
        $response = [];
        try {
            $user = Authentication::getuser();
            if($user->clpw == md5(MD5_HASH_KEY . $post['vOldPassword'])){
                $user->clpw = md5(MD5_HASH_KEY . $post['vNewPassword']);
                $user->save();
                $response = ['status' => true, 'message' => 'password_changed'];
            }else{
                $response = ['status' => false, 'message' => 'old_password_wrong'];
            }
        } catch (Exception $ex) {
            $response = ['status' => false, 'message' => $ex->getMessage()];
        }
        return $response;
    }

}
